#include "win.h"

#include <QtGui>
#include "math/primitives/polygon.h"
#include "ui/chainplug.h"
#include "ui/renderview.h"
#include "ui/stripview.h"
#include "model/activeproject.h"

Win::Win(const Environment& env, const ArcList& arcs,
		 const ColorSrcList& clrs, ActiveProject& prj,
		 QWidget *parent)
	: QWidget(parent)
	, mEnv(env)
	, mArcs(arcs)
	, mClrs(clrs)
	, mPrj(prj)
	, mStrip0(NULL)
	, mStrip1(NULL)
{
	// XXX I don't think I can rely on font info in the constructor.
	// It's usually fine, but occasionally reports garbage values.
	// I'm thinking the guarantee is after QWidget::ensurePolish()
	// has been called, which I don't believe can be done in the constructor.
	// See that method description for how to handle code after it's
	// called, although for now I'm going through the app, which I'm
	// hoping works fine.
//	const QFontMetrics		fm = fontMetrics();
	const QFontMetrics		fm = qApp->fontMetrics();
	mLook.mFh = fm.height();
	mLook.mFa = fm.ascent();
	mLook.mFd = fm.descent();

	mStrip0 = new StripView(prj, arcs, mLook);
	mStrip1 = new StripView(prj, arcs, mLook);
	mRenderView = new RenderView(prj);

	QObject::connect(&prj, SIGNAL(projectChanged(const int)),
					 mStrip1, SLOT(onProjectChanged(const int)));
	QObject::connect(&prj, SIGNAL(projectChanged(const int)),
					 mRenderView, SLOT(onProjectChanged(const int)));

	qRegisterMetaType<Param>("Param");
	QObject::connect(&prj, SIGNAL(paramChanged(const Param&)),
					 mStrip1, SLOT(onParamChanged(const Param&)));
	QObject::connect(&prj, SIGNAL(paramChanged(const Param&)),
					 mRenderView, SLOT(onParamChanged(const Param&)));

	QGridLayout *mainLayout = new QGridLayout;
//	mainLayout->addWidget(nameLabel, 0, 0);
	if (mStrip0 != NULL) {
		mStrip0->setPlug(new ChainPlug(prj, mNavigation));
		mainLayout->addWidget(mStrip0, 0, 0);
	}
	if (mStrip1 != NULL) {
		mainLayout->addWidget(mStrip1, 1, 0);
	}
//	mainLayout->addWidget(addressLabel, 2, 0, Qt::AlignTop);
	mainLayout->addWidget(mRenderView, 2, 0);

	setLayout(mainLayout);
	setWindowTitle(tr("Arclight"));

	QObject::connect(&mNavigation, SIGNAL(onSelectFx(const int)),
					 this, SLOT(onSelectFx(const int)));
}

/* NEW-FX-UI.  Given an FX id and
 * a Create, create a new UI.
 *******************************/
class Win::NewFxUi : public fx::Visitor {
public:
	const int		mFxId;
	FxUi::Create	mCreate;
	FxUi*			mUi;

	NewFxUi(const int fxId, const Environment& env, ActiveProject& prj,
			const ArcList& arcs, const ColorSrcList& clrs, const int mode)
		: mFxId(fxId)
		, mCreate(env, prj, arcs, clrs, mode)
		, mUi(NULL)
	{
	}

	virtual int		visit(Fx& fx, fx::VisitParams&)
	{
		if (fx.getId() != mFxId) return fx::CONTINUE;
		mUi = fx.newUi(mCreate);
		return fx::STOP;
	}
};

void Win::onSelectFx(const int fxId)
{
	if (mStrip1 == NULL) return;
	NewFxUi		p(fxId, mEnv, mPrj, mArcs, mClrs, FxUi::EDIT);
	mPrj.forEach(p);

	mStrip1->setPlug(p.mUi);
}

