#include "arc_list.h"

#include "arc/arcvisitors.h"
#include "engine/arc.h"

/* ARC-LIST
 ******************************************************************/
ArcList::ArcList()
{
}

ArcList::~ArcList()
{
	while (!mE.isEmpty()) delete mE.takeFirst();
}

bool ArcList::getArc(const int idx, ArcHead& head, QString* name) const
{
	if (idx < 0 || idx >= mE.size()) return false;
	const Entry*		e = mE[idx];
	if (e == NULL) return false;
	head.setTo(e->mArc);
	if (name != NULL) *name = e->mName;
	return true;
}

bool ArcList::add(Entry* e)
{
	if (e == NULL) return false;
	Arc*		a = e->mArc.getArc();
	if (a != NULL) {
		// Make sure all the arcs have IDs set
		arc::LastId		last;
		a->forEach(last);
		arc::FillId		fill(last.mLastId+199);
		a->forEach(fill);
	}
	mE.append(e);
	return true;
}

/* ENTRY
 ******************************************************************/
ArcList::Entry::Entry()
{
}

/* DEFAULTS
 ******************************************************************/
#include "arc/arc_source.h"
#include "arc/arcchain.h"
#include "arc/binary.h"
#include "arc/constant.h"
#include "arc/noise.h"
#include "arc/pow.h"
#include "arc/sine.h"

void ArcList::addDefaults(ArcList& list)
{
	// NOTE: AFTER adding the entry to the list, which assigns
	// the arc IDs, we fill in the table.
	Entry*		e;
	// Old-style modulation types, before adding the arc::Source.
	// Think these can (and should) completely go away.
#if 0
	if ((e = new Entry()) != NULL) {
		e->mName = "Distance";
		Arc*			c = new arc::Constant(0);
		e->mArc.setArc(c);
		list.add(e);	// assign arc IDs
		e->mArc.addLink(arc::ModLink(arc::DISTANCE_SRC, c->getId(), 0, 0, 1));
	}
	if ((e = new Entry()) != NULL) {
		e->mName = "Normal";
		Arc*			c = new arc::Constant(0);
		e->mArc.setArc(c);
		list.add(e);	// assign arc IDs
		e->mArc.addLink(arc::ModLink(arc::NORMAL_SRC, c->getId(), 0));
	}
	if ((e = new Entry()) != NULL) {
		e->mName = "D*N";
		Arc*			lc = new arc::Constant(0);
		Arc*			rc = new arc::Constant(0);
		e->mArc.setArc(new arc::Binary(arc::Binary::OP_MULT, lc, rc));
		list.add(e);	// assign arc IDs
		e->mArc.addLink(arc::ModLink(arc::DISTANCE_SRC, lc->getId(), 0, 0, 1));
		e->mArc.addLink(arc::ModLink(arc::NORMAL_SRC, rc->getId(), 0));
	}
	if ((e = new Entry()) != NULL) {
		e->mName = "sine D*N";
		Arc*			lc = new arc::Constant(0);
		Arc*			rc = new arc::Constant(0);
		arc::Chain*		chain = new arc::Chain();
		chain->append(lc);
		chain->append(new arc::Sine(1, 0));
		e->mArc.setArc(new arc::Binary(arc::Binary::OP_MULT, chain, rc));
		list.add(e);	// assign arc IDs
		e->mArc.addLink(arc::ModLink(arc::DISTANCE_SRC, lc->getId(), 0, 0, 1));
		e->mArc.addLink(arc::ModLink(arc::NORMAL_SRC, rc->getId(), 0));
	}
	if ((e = new Entry()) != NULL) {
		e->mName = "(sine D*N) nse";
		Arc*			lc = new arc::Constant(0);
		Arc*			rc = new arc::Constant(0);
		arc::Chain*		chain = new arc::Chain();
		chain->append(lc);
		chain->append(new arc::Sine(1, 0));
		chain->append(new arc::Noise(0.05f));
		e->mArc.setArc(new arc::Binary(arc::Binary::OP_MULT, chain, rc));
		list.add(e);	// assign arc IDs
		e->mArc.addLink(arc::ModLink(arc::DISTANCE_SRC, lc->getId(), 0, 0, 1));
		e->mArc.addLink(arc::ModLink(arc::NORMAL_SRC, rc->getId(), 0));
	}
#endif

	// Using the new, source-based, non-modulation arcs
	if ((e = new Entry()) != NULL) {
		e->mName = "D";
		e->mArc.setArc(new arc::Source(arc::DISTANCE_SRC));
		list.add(e);	// assign arc IDs
	}
	if ((e = new Entry()) != NULL) {
		e->mName = "N";
		e->mArc.setArc(new arc::Source(arc::NORMAL_SRC));
		list.add(e);	// assign arc IDs
	}
	if ((e = new Entry()) != NULL) {
		e->mName = "D*N";
		Arc*			lc = new arc::Source(arc::DISTANCE_SRC);
		Arc*			rc = new arc::Source(arc::NORMAL_SRC);
		e->mArc.setArc(new arc::Binary(arc::Binary::OP_MULT, lc, rc));
		list.add(e);	// assign arc IDs
	}
	if ((e = new Entry()) != NULL) {
		e->mName = "((D sine)*N) nse";
		arc::Chain*		lc = new arc::Chain();
		lc->append(new arc::Source(arc::DISTANCE_SRC));
		lc->append(new arc::Sine(1, 0, true));
// Need to add a saturation arc
//lc->append(new arc::Constant(2));
		arc::Chain*		root = new arc::Chain();
		root->append(new arc::Binary(arc::Binary::OP_MULT, lc, new arc::Source(arc::NORMAL_SRC)));
		root->append(new arc::Noise(0.05f));
		e->mArc.setArc(root);
		list.add(e);	// assign arc IDs
	}
	// Let's try some modulations

	// Distance modulated by normal, normal modulated by distance
	if ((e = new Entry()) != NULL) {
		e->mName = "Dn*Nd";
		arc::Chain*		lc = new arc::Chain();
		arc::Chain*		rc = new arc::Chain();
		lc->append(new arc::Source(arc::DISTANCE_SRC));
		rc->append(new arc::Source(arc::NORMAL_SRC));
		Arc*			Dn = new arc::Constant(0.5f);
		Arc*			Nd = new arc::Constant(0.5f);
		lc->append(Dn);
		rc->append(Nd);
		e->mArc.setArc(new arc::Binary(arc::Binary::OP_MULT, lc, rc));
		list.add(e);	// assign arc IDs
		e->mArc.addLink(arc::ModLink(arc::NORMAL_SRC, Dn->getId(), 0, 0.5f, 1));
		e->mArc.addLink(arc::ModLink(arc::DISTANCE_SRC, Nd->getId(), 0, 0.5f, 1));
	}

	if ((e = new Entry()) != NULL) {
		e->mName = "pow(D)";
		arc::Chain*		c = new arc::Chain();
		c->append(new arc::Source(arc::DISTANCE_SRC));
		c->append(new arc::Pow());
		e->mArc.setArc(c);
		list.add(e);	// assign arc IDs
	}

	// Sine modulated by normal
	// Eh stupid
/*
	if ((e = new Entry()) != NULL) {
		e->mName = "D*(sine N)";
		Arc*			l = new arc::Source(arc::DISTANCE_SRC);
		Arc*			r = new arc::Sine(1, 0);
		e->mArc.setArc(new arc::Binary(arc::Binary::OP_MULT, l, r));
		list.add(e);	// assign arc IDs
		e->mArc.addLink(arc::ModLink(arc::NORMAL_SRC, r->getId(), 0, 0, 0.25f));
	}
*/
}
