#include "arcchain.h"

#include <stdlib.h>

using namespace arc;

static const QString		ARC_NAME("Chain");

/* CHAIN
 ******************************************************************/
Chain::Chain()
{
}

Chain::Chain(const Chain& o)
	: Arc(o)
{
	for (int k=0; k<o.mArc.size(); k++) {
		Arc*		a = o.mArc[k];
		if (a != NULL && (a = a->copy()) != NULL) mArc.append(a);
	}
}

Chain::~Chain()
{
	while (!mArc.isEmpty()) delete mArc.takeFirst();
}

Chain* Chain::copy() const
{
	return new Chain(*this);
}

void Chain::getParams(param::List& list, param::GetArgs& get) const
{
	for (int k=0; k<mArc.size(); k++) {
		if (mArc[k] == NULL) continue;

		const Arc&		arc = *(mArc[k]);
		param::List		l;
		arc.getParams(l, get);
		l.prepend(arc.getId());
		list.takeAll(l);
	}
}

param::Parent* Chain::findParent(param::UrlIterator& it)
{
	for (int k=0; k<mArc.size(); k++) {
		Arc*			a = mArc[k];
		if (a == NULL || !it.matches(a->getId())) continue;

		++it;
		return a->findParent(it);
	}
	return NULL;
}

void Chain::setParam(const param::UrlIterator&, param::SetArgs&)
{
}

int Chain::forEach(Visitor& v)
{
	v.visit(*this);

	for (int k=0; k<mArc.size(); k++) {
		if (mArc[k] == NULL) continue;
		const int	ans = mArc[k]->forEach(v);
		if (ans < arc::CONTINUE) return ans;
	}
	return arc::CONTINUE;
}

void Chain::work(WorkParams& p, float* row)
{
	for (int k=0; k<mArc.size(); k++) {
		Arc*		a = mArc[k];
		if (a != NULL) a->work(p, row);
	}
}

void Chain::append(Arc* a)
{
	if (a != NULL) mArc.append(a);
}

#ifdef QT_DEBUG
void Chain::printTree(const int tabs) const
{
	{
		QDebug		d = qDebug();
		for (int k=0; k<tabs; k++) (d.nospace()) << "\t";
		(d.nospace()) << ARC_NAME;
		(d.nospace()) << " id=";
		(d.nospace()) << getId();
	}
	for (int k=0; k<mArc.size(); k++) {
		if (mArc[k] == NULL) continue;

		const Arc&		a = *(mArc[k]);
		a.printTree(tabs+1);
	}
}
#endif
