#include "constant.h"

#include <assert.h>

using namespace arc;

static const QString		ARC_NAME("Constant");
static const QString		AMOUNT_NAME("Amount");
static const int			AMOUNT_ID = 0;

/* CONSTANT
 ******************************************************************/
Constant::Constant(const float v)
	: mV(v)
{
}

Constant::Constant(const Constant& o)
	: Arc(o)
	, mV(o.mV)
{
}

Constant* Constant::copy() const
{
	return new Constant(*this);
}

void Constant::getParams(param::List& list, param::GetArgs& get) const
{
	list.addParam(&ARC_NAME, AMOUNT_NAME, AMOUNT_ID, mV, get);
}

param::Parent* Constant::findParent(param::UrlIterator& it)
{
	assert(it.hasValue());
	if (!it.hasValue()) return NULL;
	return this;
}

void Constant::setParam(const param::UrlIterator& it, param::SetArgs& sa)
{
	if (it.matches(AMOUNT_ID)) {
		if (mV == sa.mV) return;	// no change
		mV = sa.mV;
	} else {
		return;
	}
	sa.mChanged = true;
}

void Constant::work(WorkParams& p, float* row)
{
	float		v = mV;
	ModEval*	mod = p.acquireEval(this, 0, &v);

	for (int k=0; k<p.mRowSize; k++) {
		if (mod != NULL) mod->moveTo(k);

		row[k] *= v;
	}

	if (mod != NULL) p.releaseEval(mod);
}

#ifdef QT_DEBUG
void Constant::printTree(const int tabs) const
{
	QDebug		d = qDebug();
	for (int k=0; k<tabs; k++) (d.nospace()) << "\t";

	(d.nospace()) << ARC_NAME;
	(d.nospace()) << " id=";
	(d.nospace()) << mId;
	(d.nospace()) << " v=";
	(d.nospace()) << mV;
}
#endif
