#include "noise.h"

#include <assert.h>

using namespace arc;

static const QString		ARC_NAME("Noise");
static const QString		AMOUNT_NAME("Noise");
static const int			AMOUNT_ID = 0;

/* NOISE
 ******************************************************************/
Noise::Noise(const float amount)
	: mAmt(amount)
{
}

Noise::Noise(const Noise& o)
	: Arc(o)
	, mAmt(o.mAmt)
{
}

Noise* Noise::copy() const
{
	return new Noise(*this);
}

void Noise::getParams(param::List& list, param::GetArgs& get) const
{
	list.addParam(&ARC_NAME, AMOUNT_NAME, AMOUNT_ID, mAmt, get);
}

param::Parent* Noise::findParent(param::UrlIterator& it)
{
	assert(it.hasValue());
	if (!it.hasValue()) return NULL;
	return this;
}

void Noise::setParam(const param::UrlIterator& it, param::SetArgs& sa)
{
	if (it.matches(AMOUNT_ID)) {
		if (mAmt == sa.mV) return;	// no change
		mAmt = sa.mV;
	} else {
		return;
	}
	sa.mChanged = true;
}

void Noise::work(WorkParams& p, float* row)
{
	float		amt = mAmt;
	ModEval*	mod = p.acquireEval(this, 0, &amt);

	for (int k=0; k<p.mRowSize; k++) {
		if (mod != NULL) mod->moveTo(k);

		row[k] += (mLin.NextRel() * amt);
	}

	if (mod != NULL) p.releaseEval(mod);
}

#ifdef QT_DEBUG
void Noise::printTree(const int tabs) const
{
	QDebug		d = qDebug();
	for (int k=0; k<tabs; k++) (d.nospace()) << "\t";

	(d.nospace()) << ARC_NAME;
	(d.nospace()) << " id=";
	(d.nospace()) << mId;
}
#endif
