#include "pow.h"

#include <assert.h>
#include <math.h>

using namespace arc;

static const QString		ARC_NAME("Sine");
static const QString		AMT_NAME("Pow");
static const int			AMT_ID = 0;

static const float			AMT_MAX = 10;

/* POW
 ******************************************************************/
Pow::Pow(const float amt)
	: mAmt(amt)
{
}

Pow::Pow(const Pow& o)
	: Arc(o)
	, mAmt(o.mAmt)
{
}

Pow* Pow::copy() const
{
	return new Pow(*this);
}

void Pow::getParams(param::List& list, param::GetArgs& get) const
{
	list.addParam(&ARC_NAME, AMT_NAME, AMT_ID, mAmt / AMT_MAX, get);
}

param::Parent* Pow::findParent(param::UrlIterator& it)
{
	assert(it.hasValue());
	if (!it.hasValue()) return NULL;
	return this;
}

void Pow::setParam(const param::UrlIterator& it, param::SetArgs& sa)
{
	if (it.matches(AMT_ID)) {
		float		trans = sa.mV * AMT_MAX;
		if (trans < 0) trans = 0;
		else if (trans > AMT_MAX) trans = AMT_MAX;
		if (mAmt == trans) return;		// no change
		mAmt = trans;
	} else {
		return;
	}
	sa.mChanged = true;
}

void Pow::work(WorkParams& p, float* row)
{
	float		amt = mAmt;
	ModEval*	mod = p.acquireEval(this, 0, &amt);

	float				v;
	for (int k=0; k<p.mRowSize; k++) {
		if (mod != NULL) mod->moveTo(k);

		row[k] = powf(row[k], amt);
	}

	if (mod != NULL) p.releaseEval(mod);
}

#ifdef QT_DEBUG
void Pow::printTree(const int tabs) const
{
	QDebug		d = qDebug();
	for (int k=0; k<tabs; k++) (d.nospace()) << "\t";

	(d.nospace()) << ARC_NAME;
	(d.nospace()) << " id=";
	(d.nospace()) << mId;
	(d.nospace()) << " amt=";
	(d.nospace()) << mAmt;
}
#endif
