#include "colorsrc1.h"

#include <QColor>

#include "engine/engineparam.h"

static const QString		NAME("Color");
static const QString		R_NAME("R");
static const int			R_ID = 0;
static const QString		G_NAME("G");
static const int			G_ID = 1;
static const QString		B_NAME("B");
static const int			B_ID = 2;
static const QString		A_NAME("A");
static const int			A_ID = 3;

static void assign(param::SetArgs& sa, float& dst, const float a);

/* COLOR-SRC-1
 ******************************************************************/
ColorSrc1::ColorSrc1(const int r, const int g, const int b, const int a)
{
	mClr.setRgba(r, g, b, a);
}

ColorSrc1::ColorSrc1(const ColorSrc1& o)
	: ColorSrc(o)
	, mClr(o.mClr)
{
}

ColorSrc1* ColorSrc1::copy() const
{
	return new ColorSrc1(*this);
}

void ColorSrc1::get(const float v, color_f& output) const
{
	output = mClr;
	// The input value determines the opacity.
	output.setA(mClr.a * v);
}

void ColorSrc1::scaleAlpha(const float a)
{
	mClr.setA(mClr.a * a);
}

void ColorSrc1::getParams(param::List& list, param::GetArgs& get) const
{
	// Convert my premultipled color into 8-bit unmatted values
	// (although I still communicate with floats of 0-1).
	QColor			clr;
	mClr.getColor(clr);
	list.addParam(&NAME, R_NAME, R_ID, clr.redF(), get);
	list.addParam(&NAME, G_NAME, G_ID, clr.greenF(), get);
	list.addParam(&NAME, B_NAME, B_ID, clr.blueF(), get);
	list.addParam(&NAME, A_NAME, A_ID, mClr.a, get);
}

void ColorSrc1::setParam(const param::UrlIterator& it, param::SetArgs& sa)
{
	if (it.matches(R_ID))			assign(sa, mClr.r, mClr.a);
	else if (it.matches(G_ID))		assign(sa, mClr.g, mClr.a);
	else if (it.matches(B_ID))		assign(sa, mClr.b, mClr.a);
	else if (it.matches(A_ID))		assign(sa, mClr.a, 1);
}

/* Misc
 ******************************************************************/
static void assign(param::SetArgs& sa, float& dst, const float a)
{
	float			c = sa.mV * a;				// premultiply
	if (c < 0) c = 0; else if (c > 1) c = 1;	// limit
	if (dst != c) {
		dst = c;
		sa.mChanged = true;
	}
}
