#include "colorsrc2.h"

#include <QColor>

#include "engine/engineparam.h"

static const QString		NAME("Color");
// clr0
static const QString		R1_NAME("R1");
static const int			R1_ID = 0;
static const QString		G1_NAME("G1");
static const int			G1_ID = 1;
static const QString		B1_NAME("B1");
static const int			B1_ID = 2;
static const QString		A1_NAME("A1");
static const int			A1_ID = 3;
// clr1
static const QString		R2_NAME("R2");
static const int			R2_ID = 4;
static const QString		G2_NAME("G2");
static const int			G2_ID = 5;
static const QString		B2_NAME("B2");
static const int			B2_ID = 6;
static const QString		A2_NAME("A2");
static const int			A2_ID = 7;

static void assign(param::SetArgs& sa, float& dst, const float a);

/* COLOR-SRC-2
 ******************************************************************/
ColorSrc2::ColorSrc2()
{
	mClr0.setRgba(0, 0, 0, 255);
	mClr1.setRgba(255, 255, 255, 255);
}

ColorSrc2::ColorSrc2(const ColorSrc2& o)
	: ColorSrc(o)
	, mClr0(o.mClr0)
	, mClr1(o.mClr1)
{
}

ColorSrc2* ColorSrc2::copy() const
{
	return new ColorSrc2(*this);
}

void ColorSrc2::get(const float v, color_f& output) const
{
	const float			left = 1-v;
	output.r = (mClr0.r*left) + (mClr1.r*v);
	output.g = (mClr0.g*left) + (mClr1.g*v);
	output.b = (mClr0.b*left) + (mClr1.b*v);
	output.a = (mClr0.a*left) + (mClr1.a*v);
}

void ColorSrc2::scaleAlpha(const float a)
{
	mClr0.setA(mClr0.a * a);
	mClr1.setA(mClr1.a * a);
}

void ColorSrc2::getParams(param::List& list, param::GetArgs& get) const
{
	// Convert my premultipled color into 8-bit unmatted values
	// (although I still communicate with floats of 0-1).
	QColor			clr;
	mClr0.getColor(clr);
	list.addParam(&NAME, R1_NAME, R1_ID, clr.redF(), get);
	list.addParam(&NAME, G1_NAME, G1_ID, clr.greenF(), get);
	list.addParam(&NAME, B1_NAME, B1_ID, clr.blueF(), get);
	list.addParam(&NAME, A1_NAME, A1_ID, mClr0.a, get);
	mClr1.getColor(clr);
	list.addParam(&NAME, R2_NAME, R2_ID, clr.redF(), get);
	list.addParam(&NAME, G2_NAME, G2_ID, clr.greenF(), get);
	list.addParam(&NAME, B2_NAME, B2_ID, clr.blueF(), get);
	list.addParam(&NAME, A2_NAME, A2_ID, mClr1.a, get);
}

void ColorSrc2::setParam(const param::UrlIterator& it, param::SetArgs& sa)
{
	if (it.matches(R1_ID))			assign(sa, mClr0.r, mClr0.a);
	else if (it.matches(G1_ID))		assign(sa, mClr0.g, mClr0.a);
	else if (it.matches(B1_ID))		assign(sa, mClr0.b, mClr0.a);
	else if (it.matches(A1_ID))		assign(sa, mClr0.a, 1);
	else if (it.matches(R2_ID))		assign(sa, mClr1.r, mClr1.a);
	else if (it.matches(G2_ID))		assign(sa, mClr1.g, mClr1.a);
	else if (it.matches(B2_ID))		assign(sa, mClr1.b, mClr1.a);
	else if (it.matches(A2_ID))		assign(sa, mClr1.a, 1);
}

/* Misc
 ******************************************************************/
static void assign(param::SetArgs& sa, float& dst, const float a)
{
	float			c = sa.mV * a;				// premultiply
	if (c < 0) c = 0; else if (c > 1) c = 1;	// limit
	if (dst != c) {
		dst = c;
		sa.mChanged = true;
	}
}
