#include "colorsrclist.h"

#include "compositing/colorsrc1.h"
#include "compositing/colorsrc2.h"
#include "engine/fxdefs.h"

const char*			COLOR_SRC_DEFAULT_SOLO = "arp:Solo";
const char*			COLOR_SRC_DEFAULT_DUO = "arp:Duo";

/* COLOR-SRC-LIST
 ******************************************************************/
ColorSrcList::ColorSrcList()
{
}

ColorSrcList::~ColorSrcList()
{
	while (!mList.isEmpty()) delete mList.popFront();
}

bool ColorSrcList::find(ColorSrcHead& h, const ColorSrcMatch& m) const
{
	const ColorSrcHead*			found = NULL;
	float						foundQ = -1;
	RecycleListIterator<ColorSrcHead*>		it(mList);
	while (it.hasValue()) {
		const ColorSrcHead*		f = *it;
		if (f != NULL) {
			const float			q = m.quality(*f);
			if (q > 0 && q > foundQ) {
				found = f;
				foundQ = q;
			}
	   }
		++it;
	}
	if (found == NULL) {
		m.getDefault(h);
		return false;
	}
	h = *found;
	return true;
}

void ColorSrcList::forEach(Visitor& v) const
{
	RecycleListIterator<ColorSrcHead*>		it(mList);
	while (it.hasValue()) {
		const ColorSrcHead*		h = *it;
		if (h != NULL) {
			if (v.visit(*h) < fx::CONTINUE) return;
	   }
		++it;
	}
}

const ColorSrcHead* ColorSrcList::getAt(int idx) const
{
	RecycleListIterator<ColorSrcHead*>		it(mList);
	while (it.hasValue()) {
		const ColorSrcHead*		h = *it;
		if (h != NULL && idx == 0) return h;
	   --idx;
		++it;
	}
	return NULL;
}

/* COLOR-SRC-LIST defaults
 ******************************************************************/
static void add(RecycleList<ColorSrcHead*>& list, ColorSrcHead* h)
{
	if (h == NULL || h->getSrc() == NULL || !list.pushFront(h)) delete h;
}

void ColorSrcList::addDefaults(ColorSrcList& list)
{
	add(list.mList, new ColorSrcHead(COLOR_SRC_DEFAULT_DUO, new ColorSrc2()));
	add(list.mList, new ColorSrcHead(COLOR_SRC_DEFAULT_SOLO, new ColorSrc1(0, 0, 0, 255)));
}
