#ifndef ARCHEAD_H
#define ARCHEAD_H

#include "engine/arcmod.h"
#include "engine/parent.h"

class Arc;

namespace arc {
	class Visitor;
	class WorkParams;
}

/* ARC-HEAD
 * Store a single arc.  Convenience so clients don't
 * have to memory manage.
 ******************************************************************/
class ArcHead : public param::Parent
{
public:
    ArcHead();
	ArcHead(const ArcHead&);
	~ArcHead();

	bool				isEmpty() const;

	// The ID for a head is generally set by the object that contains
	// it, as a way to uniquely identify it from other parameters.
	void				setId(const int);

	Arc*				getArc();
	void				setArc(Arc*);
	void				setTo(const ArcHead&);
	// Used by clients to set the contents, leaving the ID alone.
	void				setContentTo(const ArcHead&);

	param::Parent*		findParent(param::UrlIterator&);

	void				getParams(param::List&, param::GetArgs&) const;
	virtual void		setParam(const param::UrlIterator&, param::SetArgs&);

	void				addLink(const arc::ModLink&);

	int					forEach(arc::Visitor&);

#ifdef QT_DEBUG
	void				printTree(const int tabs) const;
#endif

protected:
	friend class arc::WorkParams;

	arc::ModTable		mLinks;

private:
	void				operator=(const ArcHead&);

	Arc*				mArc;
};

/* inline
 ******************************************************************/
inline bool ArcHead::isEmpty() const
{
	return mArc == NULL;
}

#endif // ARCHEAD_H
