#include "colorf.h"

#include <QColor>

// Macros for accessing colour elements in the dst array during compositing
#define PIXR(pix)		(pix)
#define PIXG(pix)		(pix+1)
#define PIXB(pix)		(pix+2)
#define PIXA(pix)		(pix+3)

/* COLOR-F
 ******************************************************************/
color_f::color_f()
	: r(0)
	, g(0)
	, b(0)
	, a(0)
{
}

color_f::color_f(const color_f& o)
	: r(o.r)
	, g(o.g)
	, b(o.b)
	, a(o.a)
{
}

color_f& color_f::operator=(const color_f& o)
{
	if (this != &o) {
		r = o.r;
		g = o.g;
		b = o.b;
		a = o.a;
	}
	return *this;
}

void color_f::setRgba(const int ru, const int gu, const int bu, const int au)
{
	a = (float)au / 255.0f;
	r = ((float)ru / 255.0f) * a;
	g = ((float)gu / 255.0f) * a;
	b = ((float)bu / 255.0f) * a;
}

void color_f::setA(const float au)
{
	const float		ratio = (au / a);
	r *= ratio;
	g *= ratio;
	b *= ratio;
	a = au;
}

static inline int expand(const float cf, const float scale)
{
	int			ci = (int)(cf * scale * 255);
	if (ci <= 0) return 0;
	if (ci >= 255) return 255;
	return ci;
}

void color_f::getColor(QColor& clr) const
{
	const float		s = 1 / a;
	clr.setRgb(expand(r, s), expand(g, s), expand(b, s), expand(a, 1));
}

/* COMPOSITING
 ******************************************************************/
void srcOver(const color_f& src, float* dst, int pix)
{
	const float		am = (1-src.a);
	pix *= 4;
	dst[PIXR(pix)] = src.r + (am*dst[PIXR(pix)]);
	dst[PIXG(pix)] = src.g + (am*dst[PIXG(pix)]);
	dst[PIXB(pix)] = src.b + (am*dst[PIXB(pix)]);
	dst[PIXA(pix)] = src.a + (am*dst[PIXA(pix)]);
}

#undef PIXR
#undef PIXG
#undef PIXB
#undef PIXA
