#ifndef COLORF_H
#define COLORF_H

class QColor;

/* COLOR-F
 * Store a color in our standard format, floating point
 * premultiplied RGBA.
 ******************************************************************/
struct color_f
{
	float			r, g, b, a;

	color_f();
	color_f(const color_f&);

	color_f&		operator=(const color_f&);

	// Set based on non-premultiplied 8-bit values
	void			setRgba(const int r, const int g, const int b, const int a);

	// Set the alpha value, maintaining the proper premultiplication
	void			setA(const float a);

	// Answer myself as a QColor
	void			getColor(QColor&) const;
};

/* COMPOSITING
 * The compositing commands taking a source colour and destination
 * image data.  THe dst is stored as floats in RGBA, but the pix
 * argument treats it like an array of single values, that is, it
 * will get multiplied by 4.  Implementations are the standard
 * Porter-Duff compositing modes.
 ******************************************************************/
void srcOver(const color_f& src, float* dst, int pix);


#endif // COLORF_H
