#include "colorsrchead.h"

#include "stdlib.h"
#include <QDebug>
#include "compositing/colorsrc1.h"
#include "compositing/colorsrclist.h"

/* COLOR-SRC-HEAD
 ******************************************************************/
ColorSrcHead::ColorSrcHead()
	: mSrc(NULL)
{
}

ColorSrcHead::ColorSrcHead(const ColorSrcHead& o)
	: Parent(o)
	, mKey(o.mKey)
	, mSrc(o.mSrc != NULL ? o.mSrc->copy() : NULL)
{
}

ColorSrcHead::ColorSrcHead(const QString& key, ColorSrc* src)
	: mKey(key)
	, mSrc(src)
{
}

ColorSrcHead::~ColorSrcHead()
{
	setSrc(NULL);
}

ColorSrcHead& ColorSrcHead::operator=(const ColorSrcHead& o)
{
	if (this != &o) {
		mKey = o.mKey;
		setSrc(o.mSrc != NULL ? o.mSrc->copy() : NULL);
	}
	return *this;
}

void ColorSrcHead::setId(const int id)
{
	mId = id;
}

QString ColorSrcHead::getKey() const
{
	return mKey;
}

const ColorSrc* ColorSrcHead::getSrc() const
{
	return mSrc;
}

void ColorSrcHead::scaleAlpha(const float a)
{
	if (mSrc != NULL) mSrc->scaleAlpha(a);
}

void ColorSrcHead::setSrc(ColorSrc* src)
{
	if (mSrc == src) return;
	delete mSrc;
	mSrc = src;
}

param::Parent* ColorSrcHead::findParent(param::UrlIterator& it)
{
	// Just a passthrough
	if (mSrc == NULL) return NULL;
	// The color sources aren't composites and don't have IDs.
//	++it;
	return mSrc->findParent(it);
}

void ColorSrcHead::getParams(param::List& list, param::GetArgs& get) const
{
	if (mSrc == NULL) return;

	param::List			l;
	mSrc->getParams(l, get);
	l.prepend(mId);
	list.takeAll(l);
}

void ColorSrcHead::setParam(const param::UrlIterator&, param::SetArgs&)
{
}

void ColorSrcHead::swap(ColorSrcHead& o)
{
	if (this != &o) {
		QString		k = mKey;	mKey = o.mKey;	o.mKey = k;
		ColorSrc*	s = mSrc;	mSrc = o.mSrc;	o.mSrc = s;
	}
}

/* COLOR-SRC-MATCH
 ******************************************************************/
ColorSrcMatch::ColorSrcMatch()
{
}

ColorSrcMatch::~ColorSrcMatch()
{
}

/* COLOR-SRC-HEAD::MATCH-DEFAULT
 ******************************************************************/
class ColorSrcHead::MatchDefault : public ColorSrcMatch {
public:
	MatchDefault(const QString& key)	 : mKey(key), mDefault(0, 0, 0, 255)	{ }

	const QString			mKey;
	const ColorSrc1			mDefault;

	virtual float			quality(const ColorSrcHead& h) const
	{
		if (h.getKey() == mKey) return 2;
		return 0;
	}

	virtual void			getDefault(ColorSrcHead& h) const
	{
		ColorSrc*			c = mDefault.copy();
		if (c != NULL) h.setSrc(c);
	}
};

/* COLOR-SRC-MATCH defaults
 ******************************************************************/
static const ColorSrcHead::MatchDefault	SOLO(COLOR_SRC_DEFAULT_SOLO);
const ColorSrcMatch&						COLOR_SRC_SOLO = SOLO;

static const ColorSrcHead::MatchDefault	DUO(COLOR_SRC_DEFAULT_DUO);
const ColorSrcMatch&						COLOR_SRC_DUO = DUO;
