#ifndef COLORSRCHEAD_H
#define COLORSRCHEAD_H

#include <QString>
#include "engine/colorsrc.h"
#include "engine/parent.h"

/* COLOR-SRC-HEAD
 * Store and manage a ColorSrc.  Kind of an odd name, as
 * a colorsrc is not a list of anything, but I'm keeping with
 * the standardized name pattern (FxHead, ArcHead).
 ******************************************************************/
class ColorSrcHead : public param::Parent
{
public:
	ColorSrcHead();
	ColorSrcHead(const ColorSrcHead&);
	ColorSrcHead(const QString& key, ColorSrc*);
	~ColorSrcHead();

	ColorSrcHead&			operator=(const ColorSrcHead&);

	// The ID for a head is generally set by the object that contains
	// it, as a way to uniquely identify it from other parameters.
	void					setId(const int);

	// ColorSrcs are identified by a unique structured string.
	// The format is "creator:name", i.e. the default Angry Red
	// Planet solo color is "arp:solo".
	QString					getKey() const;

	const ColorSrc*			getSrc() const;

	void					scaleAlpha(const float);

	param::Parent*			findParent(param::UrlIterator&);

	virtual void			getParams(param::List&, param::GetArgs&) const;
	virtual void			setParam(const param::UrlIterator&, param::SetArgs&);

	// Exchange data.  This is used by clients when setting params,
	// there's no reason to copy it just to throw the original away.
	void					swap(ColorSrcHead&);

private:
	QString					mKey;
	ColorSrc*				mSrc;

	// I take ownership
	void					setSrc(ColorSrc*);

// ColorSrc matching
public:
	class MatchDefault;
};

/* COLOR-SRC-MATCH
 * Utility object for finding a color src in the context.
 ******************************************************************/
class ColorSrcMatch {
public:
	virtual ~ColorSrcMatch();

	// Answer a quality value of 0-1.  Can also answer >= 2 for
	// "exact match, stop searching."
	virtual float			quality(const ColorSrcHead&) const = 0;

	// The match can optionally provide a default if nothing was found.
	virtual void			getDefault(ColorSrcHead&) const = 0;

protected:
	ColorSrcMatch();
};

// Some defaults.  Know your matches!  The defaults are
// assumed to be used during initialization, and will always
// set the color, even to default black if no match is found.
extern const ColorSrcMatch&		COLOR_SRC_SOLO;		 // The standard single colour
extern const ColorSrcMatch&		COLOR_SRC_DUO;		 // The standard two-color fade

#endif // COLORSRCHEAD_H
