#ifndef ENGINEPARAM_H
#define ENGINEPARAM_H

#include <QString>
#include "param/param.h"
#include "engine/archead.h"
#include "engine/colorsrchead.h"

/* A few classes that straddle the boundary between the param
 * and engine modules, adding some engine-specific functionality
 * to the param framework.  From the apps perspective, it should
 * look like these things are actually in the param module, hence
 * that's the namespace used.
 ******************************************************************/

namespace param {

static const int		ARC_BROWSER_TYPE = Param::CLIENT_TYPE;
static const int		COLOR_BROWSER_TYPE = Param::CLIENT_TYPE+1;

static const int		ARC_FEATURE = (1<<0);
static const int		COLOR_FEATURE = (1<<1);

/* PARAM::SET-ARGS.  Arguments for setParam(), keeps everything
 * bundled in case the API changes over time.
 ******************************************************************/
struct SetArgs {
	// Input
	float				mV;
	QString				mStr;
	ArcHead				mArc;
	ColorSrcHead		mClr;
	// Output
	bool				mChanged;

	SetArgs();
	SetArgs(const SetArgs&);
	void				operator=(const SetArgs&);

	void				clear();

	void				readFrom(const Param&);
	void				writeTo(Param&) const;
};

} // namespace param

#endif // ENGINEPARAM_H
