#ifndef FX_H
#define FX_H

#include <QList>
#include "fxui.h"
#include "fxdefs.h"
#include "engine/engineparam.h"

class Frame;
class FxUi;

/* FX
 * Abstract processing object.  All it does is manipulate
 * frame data.  The main processing call is work().  FX
 * objects do not need to be thread safe -- a copy exists
 * in the rendering thread.
 ******************************************************************/
class Fx : public param::Parent
{
public:

public:
	virtual ~Fx();

	virtual Fx*				copy() const = 0;

	virtual QString			getLabel() const = 0;

	// Follow the URL to find the parent at the given param.
	virtual param::Parent*	findParent(param::UrlIterator&) = 0;

	virtual int				forEach(fx::Visitor&, fx::VisitParams&);

	// The main processing entry point.  The buffers allocated
	// in the args will always be large enough to accomodate
	// the output.
	virtual void			work(fx::WorkArgs&, fx::WorkOutput&) = 0;

	// A default UI editing the parameters is provided, otherwise
	// subclasses override for a custom UI.  Note that there is
	// no default browser -- asking for it will just flip to edit mode.
	virtual FxUi*			newUi(FxUi::Create&) const;

protected:
	Fx(const int uniqueId);
	Fx(const Fx&);

private:
	Fx();
};

#endif // FX_H
