#ifndef PARENT_H
#define PARENT_H

#include "param/paramlist.h"

namespace param {

struct SetArgs;

/* PARAM::PARENT.  Interface for classes that contain params.  The
 * framework is designed this way so clients can locate a param-
 * containing parent once, then reuse it, instead of tracing the
 * URL path each time the same param changes.  For clients, this
 * class should basically look like an extension of the param
 * framework even though it relies on app-specific classes, that's
 * why it's in the param namespace.
 ******************************************************************/
class Parent {
public:
	// Utility to prepend all the ids from parent up to the root.
	static void				makeUrl(const Parent*, Url&);

public:
	virtual ~Parent();

	const Parent*			getParent() const;
	void					setParent(Parent*);

	int						getId() const;

	// Subclasses should add any params to the list
	virtual void			getParams(param::List&, param::GetArgs&) const = 0;

	// This parent should have been found previously with the iterator,
	// leaving it on a valid value for a param contained in the implementor.
	// Implementors should always set the mChanged value if something
	// actually changed, so the UI can update.
	virtual void			setParam(const UrlIterator&, SetArgs&) = 0;

protected:
	Parent();
	Parent(const Parent&);

protected:
	Parent*					mParent;
	int						mId;
};

/* inline
 ******************************************************************/
inline const Parent* Parent::getParent() const
{
	return mParent;
}

inline int Parent::getId() const
{
	return mId;
}

} // namespace param

#endif // PARENT_H
