#include "surface.h"

#include <math.h>
#include "math/geo.h"

Surface::Surface()
{
}

void Surface::set(const Quad& q)
{
	mQ.set(q);
	mL = mR = mQ.mPt[0].x;
	mT = mB = mQ.mPt[0].y;
	for (int k=1; k<Quad::SIZE; k++) {
		const Point3&		pt = mQ.mPt[k];
		mL = fmin(mL, pt.x);
		mT = fmin(mT, pt.y);
		mR = fmax(mR, pt.x);
		mB = fmax(mB, pt.y);
	}
}

bool Surface::containsCCW(const float x, const float y) const
{
	if (x < mL || x > mR || y < mT || y > mB) return false;

	// Special rule for my degenerate quads that look like tris.

	// Pointing to the inside
	const Point3&		a = mQ.mPt[0];
	const Point3&		b = mQ.mPt[1];
	const Point3&		c = mQ.mPt[2];
	const Point3&		d = mQ.mPt[3];
	if (b.sameAs2(c)) return geo::isInsideTri(a.x, a.y, b.x, b.y, d.x, d.y, x, y);

	// Pointing to the outside
	if (a.sameAs2(d)) return geo::isInsideTri(a.x, a.y, b.x, b.y, c.x, c.y, x, y);

	return mQ.containsCCW(x, y);
}
