#ifndef SURFACESHADER_H
#define SURFACESHADER_H

#include "engine/surface.h"
#include "math/quadlist.h"

/* Abstract class for root geometry shading algorithms.
 * Note that subclasses are merely responsible for
 * turning points into normal and distance info, all
 * other processing is handled elsewhere.
 */
class SurfaceShader
{
public:
	struct Params {
		float		mZ;
		Point3		mN;
	};

protected:
	SurfaceShader();
	virtual ~SurfaceShader();

public:						// API
	// Setup
	virtual bool			init(const QuadList&) = 0;

	// Find the surface at the given point.
	virtual const Surface*	find(const float x, const float y) const = 0;

	// Setup for the supplied y value.
	virtual void			beginRow(const Surface&, const float y) = 0;

	// Call beginRow() with a Y value first.
	virtual void			getAt(const float x, Params& out) const = 0;

public:
	// NORMAL
	// Do the work of generating each surface normal
	// The normal is the cross product of the vectors
	// of two non-parallel sides of the surface.
	// --------------------------------------------
	struct Normal {
		void setOn(const Quad&, Point3& nrm);

	private:
		Point3			mA, mB;
	};

};

#endif // SURFACESHADER_H
