#include "chain.h"

#include <QDebug>

using namespace fx;

static const QString		FX_NAME("Chain");

/* CHAIN
 ******************************************************************/
Chain::Chain(Context& ctx) :
		Fx(ctx.nextFxUniqueId())
{
}

Chain::Chain(const Chain& o)
	: Fx(o)
{
	for (int k=0; k<o.mFx.size(); k++) {
		const Fx*		src = o.mFx[k];
		if (src == NULL) continue;
		Fx*				dst = src->copy();
		if (dst != NULL) mFx.append(dst);
	}
}

Chain::~Chain()
{
	for (int k=0; k<mFx.size(); k++) delete mFx[k];
	mFx.clear();
}

Chain* Chain::copy() const
{
	return new Chain(*this);
}

QString Chain::getLabel() const
{
	return FX_NAME;
}

void Chain::getParams(param::List& list, param::GetArgs& get) const
{
	for (int k=0; k<mFx.size(); k++) {
		if (mFx[k] == NULL) continue;

		const Fx&		fx = *(mFx[k]);
		param::List		l;
		fx.getParams(l, get);
		l.prepend(fx.getId());
		list.takeAll(l);
	}
}

param::Parent* Chain::findParent(param::UrlIterator& it)
{
	for (int k=0; k<mFx.size(); k++) {
		Fx*			fx = mFx[k];
		if (fx == NULL) continue;

		if (it.matches(fx->getId())) {
			++it;
			return fx->findParent(it);
		}
	}
	return NULL;
}

void Chain::setParam(const param::UrlIterator&, param::SetArgs&)
{
}

int Chain::forEach(Visitor& op, VisitParams& vp)
{
	for (int i=0; i<mFx.size(); i++) {
		vp.mDepth++;
		const int		ans = (mFx[i] != NULL ? mFx[i]->forEach(op, vp) : fx::CONTINUE);
		vp.mDepth--;
		if (ans < fx::CONTINUE) return ans;
	}
	return fx::CONTINUE;
}

void Chain::work(fx::WorkArgs& a, fx::WorkOutput& o)
{
	for (int i=0; i<mFx.size(); i++) {
		if (mFx[i] != NULL) mFx[i]->work(a, o);
	}
}

void Chain::append(Fx* fx)
{
	if (fx == NULL) return;

	mFx.append(fx);
	fx->setParent(this);
}
