#ifndef LIGHT_H
#define LIGHT_H

#include "engine/arcdefs.h"
#include "engine/archead.h"
#include "engine/colorsrchead.h"
#include "engine/fx.h"
#include "engine/surfaceaccum.h"
#include "engine/surfaceneighbor.h"
#include "math/quadlist.h"
#include "model/context.h"
#include "compositing/mask.h"

class Arc;

namespace fx {

/* LIGHT
 * Given a geometry, fill in the frame Cells.
 * While this is a full FX, it's currently
 * implemented and only used as part of the
 * Bevel fx.
 ******************************************************************/
class Light : public Fx
{
public:
	explicit Light(Context&);
	Light(const Light&);
	virtual ~Light();

	virtual Light*			copy() const;

	bool					set(const QuadList&);
	void					setGlobal(const float angle);

	virtual QString			getLabel() const;

	virtual void			getParams(param::List&, param::GetArgs&) const;
	virtual param::Parent*	findParent(param::UrlIterator&);
	virtual void			setParam(const param::UrlIterator&, param::SetArgs&);

	virtual void			work(fx::WorkArgs&, fx::WorkOutput&);

private:
	// State
	float					mGlobalAngle;
	// Need to work out whether this is a separate FX or not,
	// right now it's embedded in the bevel and causing API issues.
public:
	ColorSrcHead			mClr;
	ArcHead					mArc;

private:
	// Work support
//	SurfaceAccum			mShader;
	SurfaceNeighbor			mShader;
	SurfaceShader::Params	mShaderP;
	float					mMaxZ;
	bool					mInited;

	arc::WorkParams			mArcP;
	Mask					mRowV, mRowMask;
};

} // fx

#endif // LIGHT_H
