#include "colorbrowser.h"

#include <QPixmap>
#include "model/context.h"
#include "compositing/colorsrclist.h"
#include "engine/fxdefs.h"
#include "fxui/colorsrcrender.h"

/* COLOR-BROWSER::RENDER
 * The visitor that renders each color source in the list and
 * adds the results to the browser
 ******************************************************************/
class ColorBrowser::Render : public ColorSrcList::Visitor
{
public:
	Render(QList<QPixmap*>& e, const QSize& size) : mE(e), mSize(size)	{ }

	virtual int			visit(const ColorSrcHead& h) {
		QPixmap*		pm = mR.onRender(h, mSize);
		if (pm != NULL) mE.append(pm);
		return fx::CONTINUE;
	}

private:
	QList<QPixmap*>&	mE;
	const QSize			mSize;
	ColorSrcRender		mR;
};

/* COLOR-BROWSER
 ******************************************************************/
ColorBrowser::ColorBrowser(Context& prj, const ColorSrcList& cl,
							const Param& p, const QSize& iconSize)
	: mPrj(prj)
	, mClrs(cl)
	, mIconSize(iconSize)
{
	mUrl = p.mUrl;
	Render			r(mE, iconSize);
	cl.forEach(r);
}

ColorBrowser::~ColorBrowser()
{
	while (!mE.isEmpty()) delete mE.takeFirst();
}

int ColorBrowser::size() const
{
	return mE.size();
}

QSize ColorBrowser::iconSize() const
{
	return mIconSize;
}

const QPixmap* ColorBrowser::at(const int idx)
{
	if (idx < 0 || idx >= mE.size()) return NULL;
	return mE[idx];
}

void ColorBrowser::setIndex(const int idx)
{
	const ColorSrcHead*		h = mClrs.getAt(idx);
	if (h != NULL) mPrj.setParamClr(mUrl, *h);
}
