#include "colorsrcrender.h"

#include <QPixmap>
#include <QPainter>

#include "engine/colorsrchead.h"
#include "math/primitives/point3.h"

/* COLOR-SRC-RENDER
 ******************************************************************/
ColorSrcRender::ColorSrcRender()
{
}

QPixmap* ColorSrcRender::onRender(const ColorSrcHead& ch, const QSize& size)
{
	QPixmap*		ans = new QPixmap(size);
	if (ans == NULL) return NULL;

	{
		// Clear the background.
		QPainter	painter(ans);
		QPainter::CompositionMode	mode = painter.compositionMode();
		painter.setCompositionMode(QPainter::CompositionMode_Source);
		painter.fillRect(0, 0, size.width(), size.height(), QColor(0, 0, 0, 0));
		painter.setCompositionMode(mode);
	}
	if (!onRender(ch, *ans)) {
		delete ans;
		return NULL;
	}
	return ans;
}

bool ColorSrcRender::onRender(const ColorSrcHead& ch, QPixmap& pm)
{
	const ColorSrc*			src = ch.getSrc();
	if (src == NULL) return false;
	const QSize				qs = pm.size();
	if (!makeSource(qs)) return false;
	const int				w = qs.width(), h = qs.height();
	const int				size = w*h;
	const float*			mask = mMask.get(size);
	const float*			aa = mAa.get(size);

	QPainter				painter(&pm);
	QColor					qc;
	int						pix = 0;
	color_f					clr;

	for (int y=0; y<h; y++) {
		for (int x=0; x<w; x++) {
			if (aa[pix] > 0) {
				src->get(mask[pix], clr);
				if (aa[pix] < 1) clr.setA(clr.a * aa[pix]);
				clr.getColor(qc);
				painter.setPen(qc);
				painter.drawPoint(x, y);
			}
			pix++;
		}
	}
	return true;
}

bool ColorSrcRender::makeSource(const QSize& s)
{
	if (mSize == s) return true;
	const int				w = s.width(), h = s.height();
	const int				size = w * h;
	float*					mask = mMask.get(size);
	float*					aa = mAa.get(size);
	if (mask == NULL || aa == NULL) return false;

	set_to_0(mask, size);
	set_to_0(aa, size);

	const Point3			cen((w-1) * 0.5f, (h-1) * 0.5f, 0);
	Point3					cur;
	const float				maxD = (cen.x > cen.y ? cen.y : cen.x);
	float					d;
	int						pix = 0;
	for (int y=0; y<h; y++) {
		cur.y = y;
		for (int x=0; x<w; x++) {
			cur.x = x;
			d = cur.dist2(cen);
			if (d < maxD) {
				mask[pix] = 1 - (d / maxD);
				// This is the antialias
				if (d <= maxD-1) aa[pix] = 1;
				else aa[pix] = maxD - d;
			}
			pix++;
		}
	}

	mSize = s;
	return true;
}
