#ifndef PARAMEDITOR_H
#define PARAMEDITOR_H

#include "engine/fxui.h"
#include "param/param.h"

namespace fxui {

/* PARAM-EDITOR.  The default editor
 * on available params.
 ******************************************************************/
class ParamEditor : public FxUi
{
public:
	ParamEditor(Create&, const param::Url&);
	virtual ~ParamEditor();

	virtual void			setParent(Parent*, const Look&);
	virtual int				onProjectChanged();

private:
	typedef FxUi			inherited;

	Context&				mPrj;
	const ArcList&			mArcs;
	const ColorSrcList&		mClrs;
	const param::Url		mUrl;
	bool					mNeedsCache;

	// For the entry
	QPoint					mDownPt;
	float					mDownV;

	/* FX-E.  One param entry.
	 **************************************************************/
	class FxE : public Entry {
	public:
		FxE(ParamEditor&, const Param&, const QSize&, const int flags);
		virtual ~FxE();

		virtual int			flags() const { return mFlags; }
		virtual QSize		size() const;
		virtual bool		getParam(Param&) const;
		virtual void		onPaint(paint_args&);
		virtual void		mousePressEvent(const mouse_args& down);
		virtual void		mouseMoveEvent(const mouse_args& down);
		virtual void		mouseReleaseEvent(const mouse_args& down);
		virtual int			onParamChanged(const Param&);

	// Hack for the arc browser
	public:
		fxui::BrowserModel*	mBrowser;

		int					mFlags;

	private:
		ParamEditor&		mP;
		QSize				mSize;
		param::Url			mUrl;
		QString				mLabel;
		int					mParamType;
		float				mV;
	};
	QList<FxE*>				mS;

	void					cache(const int entryH, const Look&);
	void					uncache();

// Private classes
public:
	class ParamFilter;
};

} // fxui

#endif // PARAMEDITOR_H
