#include "svgcmd.h"

static bool			PRINT = false;

SvgCmd::SvgCmd(QPainterPath& p, QPointF& prev, const int size)
	: mPath(p)
	, mPrevPt(prev)
	, mIndex(0)
	, mSize(size)
	, mIsRelative(false)
	, mStack(0)
{
}

SvgCmd::~SvgCmd()
{
}

void SvgCmd::start(const bool isRelative)
{
	mIndex = 0;
	mIsRelative = isRelative;
	mStack = 0;
	if (mSize < 1) {
		onFinished();
		mStack++;
	}
}

void SvgCmd::push(const float v)
{
	mV[mIndex] = v;
	mIndex++;
	if (mIndex >= mSize) {
		if (!mIsRelative) {
			mPrevPt.setX(0);
			mPrevPt.setY(0);
		}
		onFinished();
		mIndex = 0;
		mStack++;
	}
}

/* SVG-CMD-C cubic bezier
 *********************************/
SvgCmdC::SvgCmdC(QPainterPath& path, QPointF& prev)
	: SvgCmd(path, prev, 6)
{
}

void SvgCmdC::onFinished()
{
	if (PRINT) qDebug() << "CURVE to (" << (mPrevPt.x() + mV[0]) << ", " << (mPrevPt.y() + mV[1]) << ")";

	mPath.cubicTo(	mPrevPt.x() + mV[0], mPrevPt.y() + mV[1],
					mPrevPt.x() + mV[2], mPrevPt.y() + mV[3],
					mPrevPt.x() + mV[4], mPrevPt.y() + mV[5] );
	mPrevPt.setX(mPrevPt.x() + mV[4]);
	mPrevPt.setY(mPrevPt.y() + mV[5]);
}

/* SVG-CMD-L line to
 *********************************/
SvgCmdL::SvgCmdL(QPainterPath& path, QPointF& prev)
	: SvgCmd(path, prev, 2)
{
}

void SvgCmdL::onFinished()
{
	mPrevPt.setX(mPrevPt.x() + mV[0]);
	mPrevPt.setY(mPrevPt.y() + mV[1]);
	mPath.lineTo(mPrevPt.x(), mPrevPt.y());
	if (PRINT) qDebug() << "LINE to (" << mPrevPt.x() << ", " << mPrevPt.y() << ")";
}

/* SVG-CMD-M move to
 *********************************/
SvgCmdM::SvgCmdM(QPainterPath& path, QPointF& prev)
	: SvgCmd(path, prev, 2)
{
}

void SvgCmdM::onFinished()
{
	mPrevPt.setX(mPrevPt.x() + mV[0]);
	mPrevPt.setY(mPrevPt.y() + mV[1]);
	// MOVETO commands can be stacked.  The first is the MOVETO, the rest
	// are treated as LINETOs.
	if (mStack <= 0) {
		mPath.moveTo(mPrevPt.x(), mPrevPt.y());
		if (PRINT) qDebug() << "MOVE to (" << mPrevPt.x() << ", " << mPrevPt.y() << ")";
	} else {
		mPath.lineTo(mPrevPt.x(), mPrevPt.y());
		if (PRINT) qDebug() << "MOVE (line) to (" << mPrevPt.x() << ", " << mPrevPt.y() << ")";
	}
}
