#ifndef SVGCMD_H
#define SVGCMD_H

#include <QtGui>

//class QPainterPath;

/* Support for the SvgReader.
 * SVG commands stay in effect until explicitly changed.
 * A command works by pushing values (which are always
 * just floats, as far as I know) until it's reached the
 * required number, then it exectutes.
 *******************************************************/
class SvgCmd
{
public:
	SvgCmd(QPainterPath& path, QPointF& prev, const int size);
	virtual ~SvgCmd();

	void start(const bool isRelative);
	void push(const float v);

protected:
	QPainterPath&		mPath;

	// Subclasses must fill in with the last point they set
	QPointF&			mPrevPt;
	qreal				mV[6];

	int					mIndex;
	int					mSize;
	bool				mIsRelative;

	// Commands can be stacked, and that can change the meaning -- for
	// example, multiple movetos are treated as linetos
	int					mStack;

	virtual void onFinished() = 0;
};

/* SVG-CMD-C cubic bezier
 *********************************/
class SvgCmdC : public SvgCmd {
public:
	SvgCmdC(QPainterPath& path, QPointF& prev);

protected:
	void onFinished();
};

/* SVG-CMD-L line to
 *********************************/
class SvgCmdL : public SvgCmd {
public:
	SvgCmdL(QPainterPath& path, QPointF& prev);

protected:
	void onFinished();
};

/* SVG-CMD-M move to
 *********************************/
class SvgCmdM : public SvgCmd {
public:
	SvgCmdM(QPainterPath& path, QPointF& prev);

protected:
	void onFinished();
};

#endif // SVGCMD_H
