#include <QtDebug>
#include <QtGui/QApplication>
#include "arcwindow.h"
#include "app/win.h"
#include "app/environment.h"
#include "compositing/colorsrclist.h"
#include "model/activeproject.h"
#include "compositing/mask.h"

#include "fx/bevel.h"
#include "fx/chain.h"
#include "fx/glow.h"
#include "fx/shapesrc.h"

// Couple hardcoded projects until I get file IO
static Fx* newFx(Context& ctx)
{
	fx::Chain*			c = new fx::Chain(ctx);
	if (c != NULL) {
		c->append(new fx::ShapeSrc(ctx));
		c->append(new fx::Glow(ctx, 0, 0));
		c->append(new fx::Bevel(ctx));
		c->append(new fx::Glow(ctx));
	}
	return c;
}

int main(int argc, char *argv[])
{
	// Client initialization
	init_mask();

	// All arcs in the system
	ArcList				arcs;
	ArcList::addDefaults(arcs);
	// All colorsrcs in the system
	ColorSrcList		cs;
	ColorSrcList::addDefaults(cs);

	QApplication		a(argc, argv);
	Environment			e(a);
	ActiveProject		p(arcs, cs);
	p.setFx(newFx(p));

	// Initially using QtDesigner for the top-level GUI, but I'm
	// not sure how to add custom classes to it.  On investigating
	// it looked a little painful -- like you have to write a plugin
	// that supplies class info.  Maybe I'm spoiled from Android,
	// where you just use your scoped custom class name.
//    ArcWindow w;
//    w.show();
	Win					win(e, arcs, cs, p);
	win.show();

    return a.exec();
}
