#ifndef GEO_H
#define GEO_H

class Point3;

/* GEO
 * A collection of geometry functions.
 ********************************************/

// Convert an x,y pixel value into an index in a flat array
#define IDX_FOR_PIXEL(x, y, w)		(((y)*(w))+(x))

namespace geo {

extern const float		EPS;		// a very small number
extern const float		DTOR;		// degrees to radians
extern const float		RTOD;		// radians to degrees

float slope(const float x0, const float y0, const float x1, const float y1);
float perpendicularSlope(const float x0, const float y0, const float x1, const float y1);

float dist(const float x0, const float y0, const float x1, const float y1);
// Ignore that slow square root when we're only comparing
// relative distances.
float distSquared(const float x0, const float y0, const float x1, const float y1);

void lineIntersection(	const Point3& a1, const Point3& a2,
						const Point3& b1, const Point3& b2,
						Point3& out);
void lineIntersection(	const float a1x, const float a1y,
						const float a2x, const float a2y,
						const float b1x, const float b1y,
						const float b2x, const float b2y,
						Point3& out);

bool lineSegmentIntersection(	const Point3& a1, const Point3& a2,
								const Point3& b1, const Point3& b2,
								Point3* out);
bool lineSegmentIntersection(	const float x1, const float y1,
								const float x2, const float y2,
								const float x3, const float y3,
								const float x4, const float y4,
								Point3* out);

float areaOfTri(const float ax, const float ay,
				const float bx, const float by,
				const float cx, const float cy);
bool isInsideTri(	const float ax, const float ay,
					const float bx, const float by,
					const float cx, const float cy,
					const float x, const float y);

// 0 = right, clockwise (90 = down)
float degree(const float x2, const float y2);

// Given a line in point-slope form, find a new point on
// that line that's dist away.
void newPtOnLine(	const float x, const float y, const float  m, const float dist,
					Point3& newPt);

}

#endif // GEO_H
