#include "planeplane.h"

#include "math/geo.h"

using namespace intersection;

PlanePlane::PlanePlane()
{
}

int PlanePlane::on(const Plane& p1, const Plane& p2, Segment3& L)
{
	mU.asCross3(p1.mN, p2.mN);
	const float		ax = (mU.x >= 0 ? mU.x : -mU.x),
					ay = (mU.y >= 0 ? mU.y : -mU.y),
					az = (mU.z >= 0 ? mU.z : -mU.z);

	// test if the two planes are parallel
	if ((ax+ay+az) < geo::EPS) {       // Pn1 and Pn2 are near parallel
		// test if disjoint or coincide
		mV = p2.mPt - p1.mPt;
		if (p1.mN.dot3(mV) == 0)		// Pn2.V0 lies in Pn1
			return COINCIDENT;			// Pn1 and Pn2 coincide
		else
			return DISJOINT;			// Pn1 and Pn2 are disjoint
	}

	// Pn1 and Pn2 intersect in a line
	// first determine max abs coordinate of cross product
	int      maxc;                      // max coordinate
	if (ax > ay) {
		if (ax > az)	maxc = 1;
		else			maxc = 3;
	} else {
		if (ay > az)	maxc = 2;
		else			maxc = 3;
	}

	// next, to get a point on the intersect line
	// zero the max coord, and solve for the other two
	float    d1, d2;           // the constants in the 2 plane equations
	d1 = -p1.mN.dot3(p1.mPt);  // note: could be pre-stored with plane
	d2 = -p2.mN.dot3(p2.mPt);  // ditto

	switch (maxc) {            // select max coordinate
	case 1:                    // intersect with x=0
		mIp.x = 0;
		mIp.y = (d2*p1.mN.z - d1*p2.mN.z) / mU.x;
		mIp.z = (d1*p2.mN.y - d2*p1.mN.y) / mU.x;
		break;
	case 2:                    // intersect with y=0
		mIp.x = (d1*p2.mN.z - d2*p1.mN.z) / mU.y;
		mIp.y = 0;
		mIp.z = (d2*p1.mN.x - d1*p2.mN.x) / mU.y;
		break;
	case 3:                    // intersect with z=0
		mIp.x = (d2*p1.mN.y - d1*p2.mN.y) / mU.z;
		mIp.y = (d1*p2.mN.x - d2*p1.mN.x) / mU.z;
		mIp.z = 0;
	}
	L.mPt[0] = mIp;
	L.mPt[1] = mIp + mU;
	return INTERSECT;
}
