#include "quadquad.h"

#include "math/geo.h"

using namespace intersection;

QuadQuad::QuadQuad()
{
}

int QuadQuad::on(const Quad& q1, const Quad& q2, Segment3& L)
{
	mP1.from(q1);
	mP2.from(q2);
	const int		ans = mPIP.on(mP1, mP2, L);
//qDebug() << "q2q q1=" << q1;
//qDebug() << "\tq2=" << q2;
//qDebug() << "\tANS=" << ans << " L=" << L;
#ifdef REPORT_GEO
//	mReport->mSeg.append(L);
#endif
	if (ans <= ERROR) return ans;

	// Clip the line into two segments bounded by the quads
	if (!clip(q1, L, mClip1)) return DISJOINT;
	if (!clip(q2, L, mClip2)) return DISJOINT;
//qDebug() << "\tclip1=" << mClip1;
//qDebug() << "\tclip2=" << mClip2;

	// The final line is the intersection of the two, which
	// can also be defined as the interior points of the
	// total line, which means the two points closest to
	// the opposite segment.
mOrder[0].set(&mClip1.mPt[0], 0);
mOrder[1].set(&mClip1.mPt[1], 0);
mOrder[2].set(&mClip2.mPt[0], 1);
mOrder[3].set(&mClip2.mPt[1], 1);
mOrder[0].mD = mClip2.distanceTo(mClip1.mPt[0], NULL);
mOrder[1].mD = mClip2.distanceTo(mClip1.mPt[1], NULL);
mOrder[2].mD = mClip1.distanceTo(mClip2.mPt[0], NULL);
mOrder[3].mD = mClip1.distanceTo(mClip2.mPt[1], NULL);
while (sort()) ;
/*
qDebug() << "POST SORT";
for (int k=0; k<ORDER_SIZE; k++) {
	qDebug() << k << "\td=" << mOrder[k].mD << " pt=" << (*(mOrder[k].mPt));
	if (k > 0 && mOrder[k].mD < mOrder[k-1].mD) qDebug() << "\tOUT OF ORDER";
}
*/
// To be valid, I need two non-identical pts that are on the other line.
int			dst = 0;
for (int k=0; k<ORDER_SIZE; k++) {
	if (k > 0 && mOrder[k-1].mPt->similarTo2(*(mOrder[k].mPt))) continue;
//	qDebug() << k << " d=" << mOrder[k].mD << " pt=" << (*(mOrder[k].mPt));
//	if (mOrder[k].mD < 1.0f) {
		L.mPt[dst++] = *(mOrder[k].mPt);
		if (dst >= 2) return ans;
//	}
}
return DISJOINT;

	// Find the intersection of the segments
	mOrder[0].set(&mClip1.mPt[0], 0);
	mOrder[1].set(&mClip1.mPt[1], 0);
	mOrder[2].set(&mClip2.mPt[0], 1);
	mOrder[3].set(&mClip2.mPt[1], 1);
	while (sort()) ;

	// There is one failure case -- the lines don't interweave
	// and the distance between the interior points is too great.
	if (mOrder[0].mSrc == mOrder[1].mSrc && (mOrder[2].mD - mOrder[1].mD) > 1)
		return DISJOINT;

	// Otherwise take the intersection -- my interior points
	L.mPt[0] = *(mOrder[1].mPt);
	L.mPt[1] = *(mOrder[2].mPt);
	return ans;
}

// Clip the line to the quad by finding the intersection
// on each segment, determining if that intersect point is
// in the quad, and using it if so.
bool QuadQuad::clip(const Quad& q, const Segment3& line, Segment3& out)
{
#if 0
	qDebug() << "\tCLip";
	mQIt.set(q);
	int						dst = 0;
	for (int i=1; i<mQIt.mSize; i++) {
		const Point3&		a = *(mQIt.mPt[i-1]);
		const Point3&		b = *(mQIt.mPt[i]);

		// Find the intersection between the quad's segment
		const float			d = mLIL.on(line.mPt[0], line.mPt[1], a, b);
	qDebug() << "\ta-b=" << a << "-" << b << " dist=" << d;
		if (d < 0.1f) {
	qDebug() << "\t\tn1=" << mLIL.mNearest1;
	qDebug() << "\t\tn2=" << mLIL.mNearest2;
			// Make sure the intersection pt is on the segment.
			mSeg.set(a, b);
	qDebug() << "\t\tseg d=" << mSeg.distanceTo(mLIL.mNearest1, NULL);
			if (mSeg.distanceTo(mLIL.mNearest1, NULL) < 0.01f) {
			// Make sure it lies within my shape.  Do this by determining
			// that it's actually between the points it intersected
//			const float		full = a.dist3(b);
//			if (mLIL.mNearest1.dist3(a) + mLIL.mNearest2.dist3(b) < full + 0.01f) {
				out.mPt[dst++] = mLIL.mNearest1;
				if (dst > 1) return true;
			}
		}
	}
	return false;
#else
	const Point3*			prv = &(q.mPt[Quad::SIZE-1]);
	int						dst = 0;
	for (int i=0; i<Quad::SIZE; i++) {
		const Point3*		nxt = &(q.mPt[i]);
		// This deals with degenerate quads that look like tris.
		if (prv->sameAs3(*nxt)) continue;

		// Find the intersection between the lines
		const float			d = mLIL.on(line.mPt[0], line.mPt[1], *prv, *nxt);
		if (d < 0.01f) {
			// Make sure it lies within my shape.  Do this by determining
			// that it's actually between the points it intersected
			const float		full = prv->dist3(*nxt);
			if (mLIL.mNearest1.dist3(*prv) + mLIL.mNearest2.dist3(*nxt) < full + 0.01f) {
				out.mPt[dst++] = mLIL.mNearest1;
				if (dst > 1) return true;
			}
		}
		prv = nxt;
	}
	return false;
#endif
}


bool QuadQuad::sort()
{
	bool			ans = false;
	for (int i=1; i<ORDER_SIZE; i++) {
		if (mOrder[i].mD < mOrder[i-1].mD) {
			mOrder[i].swap(mOrder[i-1]);
			ans = true;
		}
	}
	return ans;
}

/* QUAD-QUAD::ORDER
 ********************************************/
void QuadQuad::order::set(const Point3* pt, const int src)
{
	mPt = pt;
	mD = pt->length();
	mSrc = src;
}

void QuadQuad::order::swap(order& o)
{
	const Point3*		pt = mPt;	mPt = o.mPt;	o.mPt = pt;
	const float			d = mD;		mD = o.mD;		o.mD = d;
	const int			s = mSrc;	mSrc = o.mSrc;	o.mSrc = s;
}


