#ifndef QUADQUAD_H
#define QUADQUAD_H

#include "math/primitives/quad.h"
#include "math/linetoline.h"
#include "math/intersection/planeplane.h"
#include "math/mathutils.h"
#include "math/report.h"

namespace intersection {

class QuadQuad
{
public:
	static const int		INTERSECT = PlanePlane::INTERSECT;		// Success!
	static const int		ERROR = PlanePlane::ERROR;
	static const int		DISJOINT = PlanePlane::DISJOINT;
	static const int		COINCIDENT = PlanePlane::COINCIDENT;

public:
	QuadQuad();

	// Given two quads in 3d space, answer the segment that
	// intersects them, if any.
	int						on(const Quad& a1, const Quad& q2, Segment3& L);

private:
	PlanePlane				mPIP;
	Plane					mP1, mP2;
	LineToLine				mLIL;
	Segment3				mClip1, mClip2, mSeg;
	QuadIt					mQIt;

	// Used to find the intersection of the line segments
	struct order {
		const Point3*		mPt;
		float				mD;
		int					mSrc;

		void				set(const Point3*, const int src);
		void				swap(order&);
	};
	static const int		ORDER_SIZE = 4;
	order					mOrder[ORDER_SIZE];

	bool					clip(const Quad& q, const Segment3& line, Segment3& out);
	bool					sort();


#ifdef REPORT_GEO
public:
	Report*					mReport;
#endif // REPORT_GEO
};

} // intersection

#endif // QUADQUAD_H
