#ifndef RAYTRI_H
#define RAYTRI_H

#include "math/primitives/segment3.h"

namespace intersection {

class RayTri
{
public:
	RayTri();

	static const int		ERROR = -1;
	static const int		NO_INTERSECT = 0;
	static const int		INSIDE = 1;
	// The following are actual success conditions, and if
	// they're true, then out is valid
	static const int		INTERSECT_PLANE = 2;
	static const int		INTERSECT_TRI = 3;

	// When the function is done, this will answer whether or
	// not the actual segment intersected (provided the test succeeded).
	bool					mSegmentIntersects;

	// Answer a value determining the type of intersection.  Out
	// is valid on an answer of INTERSECT_PLANE or INTERSECT_TRI.
	// Tri is CCW.
	int on(	const Segment3& ray,
			const Point3& ptA, const Point3& ptB, const Point3& ptC,
			Point3& out);

	// Only use after performing the setup on a tri on the
	// same plane as this tri.
	int onTri(	const Point3& a, const Point3& b, const Point3& c,
				Point3& out);

private:
	Point3			mU, mV, mN,			// Triangle vectors
					mDir, mW0, mW;		// Ray vectors
};

} // intersection

#endif // RAYTRI_H
