#include "segseg.h"

#include "math/geo.h"

using namespace intersection;

SegSeg::SegSeg()
{
}

float SegSeg::on(	const Segment3& S1, const Segment3& S2,
					Point3* nearest)
{
	mU = S1.mPt[1] - S1.mPt[0];
	mV = S2.mPt[1] - S2.mPt[0];
	mW = S1.mPt[0] - S2.mPt[0];
	const float		a = mU.dot3(mU),		// always >= 0
					b = mU.dot3(mV),
					c = mV.dot3(mV),		// always >= 0
					d = mU.dot3(mW),
					e = mV.dot3(mW);
	const float		D = a*c - b*b;			// always >= 0
	float			sc, sN, sD = D;			// sc = sN / sD, default sD = D >= 0
	float			tc, tN, tD = D;			// tc = tN / tD, default tD = D >= 0

	// compute the line parameters of the two closest points
	if (D < geo::EPS) {		// the lines are almost parallel
		sN = 0.0;			// force using point P0 on segment S1
		sD = 1.0;			// to prevent possible division by 0.0 later
		tN = e;
		tD = c;
	} else {                // get the closest points on the infinite lines
		sN = (b*e - c*d);
		tN = (a*e - b*d);
		if (sN < 0.0) {       // sc < 0 => the s=0 edge is visible
			sN = 0.0;
			tN = e;
			tD = c;
		} else if (sN > sD) {  // sc > 1 => the s=1 edge is visible
			sN = sD;
			tN = e + b;
			tD = c;
		}
	}

	if (tN < 0.0) {           // tc < 0 => the t=0 edge is visible
		tN = 0.0;
		// recompute sc for this edge
		if (-d < 0.0)
			sN = 0.0;
		else if (-d > a)
			sN = sD;
		else {
			sN = -d;
			sD = a;
		}
	} else if (tN > tD) {      // tc > 1 => the t=1 edge is visible
		tN = tD;
		// recompute sc for this edge
		if ((-d + b) < 0.0)
			sN = 0;
		else if ((-d + b) > a)
			sN = sD;
		else {
			sN = (-d + b);
			sD = a;
		}
	}
	// finally do the division to get sc and tc
	sc = (abs(sN) < geo::EPS ? 0.0 : sN / sD);
	tc = (abs(tN) < geo::EPS ? 0.0 : tN / tD);

	// nearest
	if (nearest != NULL) *nearest = S1.mPt[0] - mU;

	// get the difference of the two closest points
//	Vector   dP = w + (sc * u) - (tc * v);
	mU *= sc;
	mV *= tc;
	mY = mU - mV;
	mDp = mW + mY;
	return mDp.length();
}
