#include "linetoline.h"

#include <QtDebug>
#include"math/geo.h"

LineToLine::LineToLine()
{
}

float LineToLine::on(const Segment3& L1, const Segment3& L2)
{
	return on(L1.mPt[0], L1.mPt[1], L2.mPt[0], L2.mPt[1]);
}

// From http://www.mathworks.com/matlabcentral/newsreader/view_thread/246420
float LineToLine::on(	const Point3& a1, const Point3& a2,
						const Point3& b1, const Point3& b2)
{
	mB2B1 = b2 - b1;
	mA1B1 = a1 - b1;
	mA2A1 = a2 - a1;

	mU.asCross3(mB2B1, mA1B1);
	mV.asCross3(mA2A1, mB2B1);
	const float		nA = mU.dot3(mV);
	mU.asCross3(mA2A1, mA1B1);
	mV.asCross3(mA2A1, mB2B1);
	const float		nB = mU.dot3(mV);
	mU.asCross3(mA2A1, mB2B1);
	mV.asCross3(mA2A1, mB2B1);
	const float		t = mU.dot3(mV);

	mA2A1 *= (nA/t);
	mNearest1 = a1 + mA2A1;
	mB2B1 *= (nB/t);
	mNearest2 = b1 + mB2B1;

//	qDebug() << "\t1=" << mNearest1 << " 2=" << mNearest2;

	return mNearest1.dist3(mNearest2);
}

// The softsurfer way.  I like it but I'm not clear on how to
// extract the nearest points
/*
	Point3		mU, mV, mW, mDp;
float LineToLine::on(	const Point3& a1, const Point3& a2,
						const Point3& b1, const Point3& b2)
{
	mU = a2 - a1;
	mV = b2 - b1;
	mW = a1 - b1;
	const float		a = mU.dot3(mU),		// always >= 0
					b = mU.dot3(mV),
					c = mV.dot3(mV),		// always >= 0
					d = mU.dot3(mW),
					e = mV.dot3(mW);
	const float		D = a*c - b*b;			// always >= 0
	float			sc, tc;

	// compute the line parameters of the two closest points
	if (D < geo::EPS) {         // the lines are almost parallel
		sc = 0.0f;
		tc = (b>c ? d/b : e/c);   // use the largest denominator
	} else {
		sc = (b*e - c*d) / D;
		tc = (a*e - b*d) / D;
	}

	// get the difference of the two closest points
//	Vector   dP = mW + (sc * mU) - (tc * mV);  // = L1(sc) - L2(tc)
	mU *= sc;
	mV *= tc;
	mDp = mW + mU - mV;

	return mDp.length();		// return the closest distance
}
*/
