#include "point3.h"

#include <math.h>

#include "math/geo.h"

Point3::Point3()
	: x(0)
	, y(0)
	, z(0)
{
}

Point3::Point3(const Point3& o)
	: x(o.x)
	, y(o.y)
	, z(o.z)
{
}

Point3::Point3(const float X, const float Y, const float Z)
	: x(X)
	, y(Y)
	, z(Z)
{
}

void Point3::set(const Point3& o)
{
	x = o.x;
	y = o.y;
	z = o.z;
}

void Point3::set(const float X, const float Y, const float Z)
{
	x = X;
	y = Y;
	z = Z;
}

bool Point3::sameAs2(const Point3& pt) const
{
	if (x == pt.x && y == pt.y) return true;
	return fabs(x-pt.x) < geo::EPS && fabs(y-pt.y) < geo::EPS;
}

bool Point3::sameAs3(const Point3& pt) const
{
	if (x == pt.x && y == pt.y && z == pt.z) return true;
	return fabs(x-pt.x) < geo::EPS && fabs(y-pt.y) < geo::EPS && fabs(z-pt.z) < geo::EPS;
}

bool Point3::similarTo2(const Point3& pt) const
{
	if (x == pt.x && y == pt.y) return true;
	return fabs(x-pt.x) < 0.01f && fabs(y-pt.y) < 0.01f;
}

void Point3::normalize()
{
	const float		nrml = sqrt((x*x) + (y*y) + (z*z));
	if (nrml != 0) *this /= nrml;
}

float Point3::length() const
{
	return sqrt(x*x + y*y + z*z);
}

float Point3::dist2(const Point3& b) const
{
	return sqrt( ((x-b.x)*(x-b.x)) + ((y-b.y)*(y-b.y)) );
}

float Point3::dist3(const Point3& b) const
{
	return sqrt( ((x-b.x)*(x-b.x)) + ((y-b.y)*(y-b.y)) + ((z-b.z)*(z-b.z)) );
}

float Point3::dot3(const Point3& v) const
{
	return x * v.x + y * v.y + z * v.z;
}

void Point3::asSub(const Point3& a, const Point3& b)
{
	x = a.x-b.x;
	y = a.y-b.y;
	z = a.z-b.z;
}

void Point3::asCross3(const Point3& a, const Point3& b)
{
	x = a.y*b.z - b.y*a.z;
	y = a.z*b.x - b.z*a.x;
	z = a.x*b.y - b.x*a.y;
}

QDebug& operator<<(QDebug& d, const Point3& pt)
{
//	((d << "(").nospace() << pt.x << ", " << pt.y).nospace() << ")";
	(d << "(").nospace();
	(d << pt.x).nospace();
	(d << ",").nospace();
	(d << pt.y).nospace();
	(d << ",").nospace();
	(d << pt.z).nospace();
	d << ")";
	return d;
}
