#ifndef POINT3_H
#define POINT3_H

#include <QtDebug>

/* POINT-3
 * Point, 3D.
 ******************************************************************/
class Point3
{
public:
    Point3();
	Point3(const Point3&);
	Point3(const float x, const float y, const float z);

	float			x, y, z;

	void set(const Point3&);
	void set(const float x, const float y, const float z);

	// A strong equivalence test, 2d
	bool sameAs2(const Point3&) const;
	bool sameAs3(const Point3&) const;

	// A weak equivalence test, 2d
	bool similarTo2(const Point3&) const;

	const Point3 operator+(const Point3& rhs) const { return Point3(x+rhs.x, y+rhs.y, z+rhs.z); }
	const Point3 operator-(const Point3& rhs) const { return Point3(x-rhs.x, y-rhs.y, z-rhs.z); }
	Point3& operator+=(const Point3& o) { x += o.x; y += o.y; z += o.z; return *this; }
	Point3& operator*=(const float& v) { x *= v; y *= v; z *= v; return *this; }
	Point3& operator/=(const float& v) { x /= v; y /= v; z /= v; return *this; }

	void normalize();
	float length() const;

	// Distance, 2d and 3d
	float dist2(const Point3&) const;
	float dist3(const Point3&) const;

	// Dot product, 3d
	float dot3(const Point3&) const;

	// Set me to the subtraction of a and b
	void asSub(const Point3& a, const Point3& b);
	// Set me to the cross product of a and b, 3d
	void asCross3(const Point3& a, const Point3& b);
};

QDebug& operator<<(QDebug&, const Point3&);

#endif // POINT3_H
