#include "range.h"

const Range			RANGE_UNI(0, 1);

/* RANGE
 ******************************************************************/
Range::Range()
	: mMin(0)
	, mMax(0)
{
}

Range::Range(const Range& o)
	: mMin(o.mMin)
	, mMax(o.mMax)
{
}

Range::Range(const float min, const float max)
	: mMin(min)
	, mMax(max)
{
}

Range& Range::operator=(const Range& o)
{
	if (this != &o) {
		mMin = o.mMin;
		mMax = o.mMax;
	}
	return *this;
}

bool Range::operator==(const Range& o) const
{
	return mMin == o.mMin && mMax == o.mMax;
}

bool Range::operator!=(const Range& o) const
{
	return mMin != o.mMin || mMax != o.mMax;
}

float Range::translateTo(float v, const Range& rf) const
{
	if (mMin == mMax || rf.mMin == rf.mMax) {
		return rf.mMin;
	}
	// Translate from source to scaled (0 - 1)
	v = (v - mMin) / (mMax - mMin);
	// Translate from scaled to dest
	v = rf.mMin + (v * (rf.mMax - rf.mMin));
	// Clip
	return rf.clip(v);
}

float Range::clip(const float v) const
{
	if (mMin < mMax) return clipForward(v);
	else if (mMin > mMax) return clipBackward(v);
	return mMin;
}

void Range::limitTo(const float v)
{
	if (v < mMin) mMin = v;
	if (v > mMax) mMax = v;
}

QDebug& operator<<(QDebug& d, const Range& r)
{
	(d << "(").nospace();
	(d << r.mMin).nospace();
	(d << " - ").nospace();
	(d << r.mMax).nospace();
	d << ")";
	return d;
}
