#ifndef RANGE_H
#define RANGE_H

#include <QtDebug>

class Range;

extern const Range		RANGE_UNI;

/* RANGE
 * Translate between value ranges.
 ******************************************************************/
class Range
{
public:
	float				mMin, mMax;

public:
    Range();
	Range(const Range&);
	Range(const float min, const float max);

	Range&				operator=(const Range&);
	bool				operator==(const Range&) const;
	bool				operator!=(const Range&) const;

	float				translateTo(float v, const Range&) const;

	float				clip(const float v) const;

	// Limit myself to the value.  Use this by setting min/max
	// to extremes (100000,-100000 or so) and then limiting each
	// value being processed, to find the final range.
	void				limitTo(const float v);

private:
	float				clipForward(const float v) const;
	float				clipBackward(const float v) const;
};

QDebug& operator<<(QDebug&, const Range&);

/*----------------------------------------------------------------*/
/*----- inline definitions ---------------------------------------*/
inline float Range::clipForward(const float v) const
{
	return (v < mMin ? mMin : (v > mMax ? mMax : v));
}

inline float Range::clipBackward(const float v) const
{
	return (v < mMax ? mMax : (v > mMin ? mMin : v));
}


#endif // RANGE_H
