#ifndef QUADLIST_H
#define QUADLIST_H

#include <QRectF>
#include "math/primitives/quad.h"

/* Store a list of quads
 **************************/
class QuadList
{
public:
	class Entry {
	public:
		Entry();

		Entry*			mNext;
		Quad			mQ;
		QRectF			mFrame;		// for faster intersection tests

		Entry*			tail();
		const Entry*	tail() const;

		bool			overlaps(const Entry&) const;

		void			cacheFrame();
	};

public:
	QuadList();
	~QuadList();

	int					size() const;

	Entry*				head();
	const Entry*		head() const;

	void				push(Entry*);
	bool				push(	const Point3& a, const Point3& b,
								const Point3& c, const Point3& d);

	void				addTail(Entry*);

	Entry*				pop();
	Entry*				pop(const Quad&);

	// Adding and removing the entire list.
	Entry*				detach();
	void				attach(Entry*);

	void				swap(QuadList&);

	// Simple utility to scale the Z values to the supplied, assuming
	// the lowest is 0.
	void				scaleToZ(const float z);

private:
	Entry*				mHead;

	void				deleteList();
};

QDebug& operator<<(QDebug&, const QuadList&);

#endif // QUADLIST_H
