#ifndef ACTIVEPROJECT_H
#define ACTIVEPROJECT_H

#include <QMutex>
#include "arc/arc_list.h"
#include "compositing/colorsrclist.h"
#include "engine/fx.h"
#include "engine/fxhead.h"
#include "model/context.h"
#include "model/project.h"
#include "param/paramlist.h"

/* ACTIVE-PROJECT
 * Store a single project and all settings needed to manipulate
 * and render it.
 *
 * The model probably isn't the best location for this -- could
 * well end up being a UI-only object.
 ******************************************************************/
class ActiveProject :	public QObject,
						public Context
{
	Q_OBJECT

public:
	// For projectChanged()
	static const int		PARAM_CHANGED = (1<<0);
	static const int		STRUCTURE_CHANGED = (1<<1);

public:
	ActiveProject(const ArcList&, const ColorSrcList&, QObject *parent = 0);
	virtual ~ActiveProject();

	virtual int				nextFxUniqueId();
	virtual bool			getArc(ArcHead&, const int idx) const;
	virtual bool			getColorSrc(ColorSrcHead&, const ColorSrcMatch&) const;

	// Set my complete FX chain
	void					setFx(Fx*);

	bool					copyProject(Project& dst) const;

	// Answer true if I have the item identified by the url
	virtual bool			contains(const param::Url&) const;

	void					forEach(fx::Visitor&);

	void					getParams(param::List&, param::GetArgs&) const;
	void					setParamF(const param::Url&, const float);
	void					setParamStr(const param::Url&, const QString&);
	void					setParamArc(const param::Url&, const ArcHead&);
	void					setParamClr(const param::Url&, const ColorSrcHead&);

signals:
	void					paramChanged(const Param&);
	void					projectChanged(const int);

private:
	// UI THREAD ONLY
	// Each new FX that gets added increments the
	// unique ID.
	int						mFxUniqueId;
	param::SetArgs			mSetParam;

	// List of all arcs in the system
	const ArcList&			mArcs;
	const ColorSrcList&		mClrSrc;

	// Every project has a unique ID, which ideally is tracked
	// by the project list, but since I don't have a project list,
	// for now I'll keep a counter that gets incremented whenever
	// the active project changes.  Which can't happen at the moment.
	int						mTimeStamp;

	// MUST BE LOCKED.  Actually, this isn't true any more.  The
	// project used to be shared between threads, but now it only
	// exists in the UI, and is copied before being sent to rendering.
	// So I *could* get rid of the locking, but for now I leave it alone.
	mutable QMutex			mLock;
	Project					mPrj;

	void					setParam(const param::Url&, param::SetArgs&, const bool structure);
};

#endif // ACTIVEPROJECT_H
