#include "paramdefs.h"

using namespace param;

/* PARAM::GET-ARGS
 ******************************************************************/
GetArgs::GetArgs()
	: mFilter(NULL)
{
}

GetArgs::~GetArgs()
{
}

bool GetArgs::supports(const int featureFlags) const
{
	if (mFilter == NULL) return false;
	return mFilter->supports(featureFlags);
}

/* PARAM::NAME-FILTER
 ******************************************************************/
class param::NameFilter : public Filter
{
public:
	NameFilter(const QString* fx, const QString* p, const int features) : mFeatures(features)
	{
		if (fx) mFx = *fx;
		if (p) mP = *p;
	}

	QString			mFx, mP;
	const int		mFeatures;

	virtual bool accept(const QString* group, const QString* param, const int) const
	{
		if (mFx.size() > 0) {
			if (group == NULL || *group != mFx) return false;
		}
		if (mP.size() > 0) {
			if (param == NULL || *param != mP) return false;
		}
		return true;
	}

	virtual bool supports(const int feature) const { return (feature&mFeatures) != 0; }
};

/* PARAM::TYPE-FILTER
 ******************************************************************/
class param::TypeFilter : public Filter
{
public:
	TypeFilter(const int type, const int features) : mType(type), mFeatures(features) { }

	const int		mType, mFeatures;

	virtual bool accept(const QString*, const QString*, const int type) const { return type == mType; }
	virtual bool supports(const int feature) const { return (feature&mFeatures) != 0; }
};

/* misc
 ******************************************************************/
Filter* newNameFilter(const QString* group, const QString* param, const int features)
{
	return new NameFilter(group, param, features);
}

Filter* newTypeFilter(const int type, const int features)
{
	return new TypeFilter(type, features);
}
