#include "paramlist.h"

#include "param.h"

using namespace param;

/* PARAM::LIST
 ******************************************************************/
List::List()
	: mHead(NULL)
{
}

List::~List()
{
	Node*		n = mHead;
	while (n != NULL) {
		Node*	next = n->mNext;
		n->mNext = NULL;
		delete n;
		n = next;
	}
}

void List::addParam(	const QString* group,
						const QString& name,
						const int paramId,
						const float fV,
						GetArgs& get)
{
	Param*			p = add(group, name, paramId, Param::FLOAT_TYPE, get);
	if (p != NULL) p->mFV = fV;
}

void List::addParam(	const QString* group,
						const QString& name,
						const int paramId,
						const QString& strV,
						GetArgs& get)
{
	Param*			p = add(group, name, paramId, Param::STRING_TYPE, get);
	if (p != NULL) p->mSV = strV;
}

void List::addTypeParam(	const QString& name,
							const int paramId,
							const int type,
							GetArgs& get)
{
	add(NULL, name, paramId, type, get);
}

void List::takeAll(List& src)
{
	if (mHead == NULL) mHead = src.mHead;
	else mHead->tail()->mNext = src.mHead;
	src.mHead = NULL;
}

void List::prepend(const int id)
{
	Node*			n = mHead;
	while (n != NULL) {
		if (n->mP != NULL) n->mP->mUrl.prepend(id);
		n = n->mNext;
	}
}

void List::prepend(const Url& url)
{
	Node*			n = mHead;
	while (n != NULL) {
		if (n->mP != NULL) n->mP->mUrl.prepend(url);
		n = n->mNext;
	}
}

Param* List::add(	const QString* group,
					const QString& name,
					const int paramId,
					const int type,
					GetArgs& get)
{
	if (get.mFilter != NULL && !get.mFilter->accept(group, &name, type)) return NULL;

	Param*			p = new Param();
	if (p == NULL) {
		get.mError = -1;
		return NULL;
	}
	if (group) p->mGroupName = *group;
	p->mParamName = name;
	p->mType = type;
	p->mUrl.prepend(paramId);

	if (mHead == NULL) mHead = new Node(p);
	else mHead->tail()->mNext = new Node(p);
	return p;
}

/* PARAM::LIST::NODE
 ******************************************************************/
List::Node::Node(Param* p)
	: mNext(NULL)
	, mP(p)
{
}

List::Node::~Node()
{
	delete mP;
}

List::Node* List::Node::tail()
{
	Node*		n = this;
	while (n->mNext != NULL) n = n->mNext;
	return n;
}

/* PARAM::LIST-ITERATOR
 ******************************************************************/
ListIterator::ListIterator()
	: mN(NULL)
{
}

ListIterator::ListIterator(const List& list)
	: mN(NULL)
{
	start(list);
}

void ListIterator::start(const List& list)
{
	mN = list.mHead;
}

void ListIterator::operator++()
{
	if (mN != NULL) mN = mN->mNext;
}

bool ListIterator::hasValue() const
{
	return mN != NULL;
}

const Param* ListIterator::operator*()
{
	if (mN == NULL) return NULL;
	return mN->mP;
}
