#ifndef PARAMLIST_H
#define PARAMLIST_H

#include <QList>
#include "param/paramdefs.h"

class Param;

namespace param {

class ListIterator;

/* PARAM::LIST
 * Store a list of params, encapsulate behaviour for manipulating
 * the list.
 ******************************************************************/
class List
{
public:
	List();
	~List();

	// MANIPULATING
	// Conveniences for adding params to lists
	void			addParam(	const QString* group, const QString& name,
								const int paramId,
								const float fV,
								GetArgs&);
	void			addParam(	const QString* group, const QString& name,
								const int paramId,
								const QString& strV,
								GetArgs&);
	// Add a param that's nothing more than a type and ID.
	void			addTypeParam(	const QString& name,
									const int paramId,
									const int type,
									GetArgs&);

	// Remove everyone from src and place them in me
	void			takeAll(List&);

	// Prepend the ID to each param URL
	void			prepend(const int id);
	void			prepend(const Url&);

private:
	friend class ListIterator;

	class Node {
	public:
		explicit Node(Param*);
		~Node();

		Node*		mNext;
		Param*		mP;

		Node*		tail();
	};
	Node*			mHead;

	Param*			add(	const QString* group,
							const QString& name,
							const int paramId,
							const int type,
							GetArgs&);
};

/* PARAM::LIST-ITERATOR
 ******************************************************************/
class ListIterator
{
public:
	ListIterator();
	ListIterator(const List&);

	void			start(const List&);
	void			operator++();

	bool			hasValue() const;
	const Param*	operator*();

private:
	List::Node*		mN;
};

} // namespace param

#endif // PARAMLIST_H
