#include "browserpopup.h"

#include <math.h>

#include <QPainter>
#include <QWidget>

#include "ui/stripview.h"
#include "arc/arc_list.h"
#include "param/param.h"
#include "model/project.h"

/* BROWSER-POP-UP
 ******************************************************************/
BrowserPopUp::BrowserPopUp(	QWidget& p, StripView&,
							fxui::BrowserModel* model)
	: mParent(p)
	, mModel(model)
	, mSize(model->iconSize())
	, mExpanded(false)
	, mPosition(0)
{
}

BrowserPopUp::~BrowserPopUp()
{
	// I don't currently own the model, but that should probably change.
//	delete mModel;
}

int BrowserPopUp::flags() const
{
	return DRAW_ON_TOP_F;
}

QSize BrowserPopUp::size() const
{
	return mSize;
}

void BrowserPopUp::onPaint(FxUi::paint_args& pa)
{
	if (mModel == NULL) return;

	if (!mExpanded) {
		const QPixmap*		pm = mModel->at(0);
		if (pm != NULL) pa.mPainter->drawPixmap(pa.mFrame.left(), pa.mFrame.top(), *pm);
	} else {
		// Draw with the current item centered on my location
		int					idx;
		float				off;
		if (getIndex(idx, off)) {
			const int		WINGS = 2;
			const int		size = mModel->size();
			const int		frameL = pa.mFrame.left();
	//		const qreal		opacity = pa.mPainter->opacity();
	//		pa.mPainter->setOpacity(0.2f);
			for (int k=-WINGS; k<=WINGS; k++) {
				const int	midx = idx+k;
				if (midx < 0 || midx >= size) continue;
				const QPixmap*		pm = mModel->at(midx);
				if (pm != NULL) {
					int	l = frameL + (off * mSize.width()) + (k*mSize.width());
					pa.mPainter->drawPixmap(l, pa.mFrame.top(), *pm);
				}
			}
//			pa.mPainter->setOpacity(opacity);
		}
	}
}

void BrowserPopUp::mousePressEvent(const FxUi::mouse_args& down)
{
	mDownPt.setX(down.mEvent->x());
	mDownPt.setY(down.mEvent->y());
	mDownV = mPosition;

	if (!mExpanded) {
		mExpanded = true;
		mParent.update();
		return;
	}
}

static float new_value(const QPoint& startPt, const int curX, const QRect& f, const float origV)
{
	const int		offset = curX - startPt.x();
	const float		newV = origV - (float(offset) / (float(f.width()*2)));
	if (newV <= 0) return 0;
	if (newV >= 1) return 1;
	return newV;
}

void BrowserPopUp::mouseMoveEvent(const FxUi::mouse_args& ma)
{
	const float		newV = new_value(mDownPt, ma.mEvent->x(), ma.mFrame, mDownV);
	mPosition = newV;
	mParent.update();
}

void BrowserPopUp::mouseReleaseEvent(const FxUi::mouse_args& down)
{
	int					idx;
	float				off;
	if (getIndex(idx, off)) {
		if (mModel != NULL) mModel->setIndex(idx);
	}
}

int BrowserPopUp::onParamChanged(const Param&)
{
	return WRONG_PARAM;
}

bool BrowserPopUp::getIndex(int& outIdx, float& outOff) const
{
	if (mModel == NULL) return false;

	const int			max = mModel->size()-1;
	float				scaled = mPosition*max;
	const float			fs = floorf(scaled);
	float				rem = scaled - fs;
	int					cen = (int)fs;
	if (rem > 0.5f) {
		cen++;
		rem = 1-(rem-0.5f)-0.5f;
	} else rem = -rem;
	if (cen < 0) cen = 0;
	else if (cen > max) cen = max;

	outIdx = cen;
	outOff = rem;
	return true;
}
