#include "chainplug.h"

#include <assert.h>

#include <QDebug>
#include <QPainter>

#include "model/activeproject.h"
#include "ui/look.h"
#include "ui/navigation.h"

/* CHAIN-PLUG::FX-CACHE-OP
 *******************************/
class ChainPlug::FxCacheOp : public fx::Visitor {
public:
	ChainPlug&		mPlug;

	FxCacheOp(ChainPlug& p) : mPlug(p) { }

	virtual int		visit(Fx& fx, fx::VisitParams& vp)
	{
		if (vp.mDepth == fx::VisitParams::ROOT_DEPTH) {
			FxE*	e = new FxE(mPlug.mNav, fx);
			if (e != NULL) mPlug.mE.append(e);
		}
		return fx::CONTINUE;
	}
};

/* CHAIN-PLUG
 *******************************/
ChainPlug::ChainPlug(ActiveProject& prj, Navigation& nav)
	: mPrj(prj)
	, mNav(nav)
	, mNeedsCache(true)
{
}

ChainPlug::~ChainPlug()
{
	uncache();
}

void ChainPlug::setParent(FxUi::Parent* p, const Look& look)
{
	assert(p != NULL);
	inherited::setParent(p, look);

	if (mNeedsCache) {
		cache(p->entryH(), look);
		mNeedsCache = false;
	}

	for (int i=0; i<mE.size(); i++) {
		if (mE[i] == NULL) continue;
		p->add(mE[i]);
	}
}

int ChainPlug::onProjectChanged()
{
	// XXX Don't really have a notion of identity for the project right now
	return FxUi::UI_UNCHANGED;
}

void ChainPlug::cache(const int entryH, const Look& look)
{
	// Cache all the top-level FX
	FxCacheOp			op(*this);
	mPrj.forEach(op);

	// Create images
	for (int i=0; i<mE.size(); i++) {
		if (mE[i] != NULL) mE[i]->cache(entryH, look);
	}
}

void ChainPlug::uncache()
{
	for (int i=0; i<mE.size(); i++) delete mE[i];
	mE.clear();
}

/* CHAIN-PLUG::FX-E
 *******************************/
ChainPlug::FxE::FxE(Navigation& nav, const Fx& fx)
	: mNav(nav)
	, mFxId(fx.getId())
	, mLabel(fx.getLabel())
{
}

QSize ChainPlug::FxE::size() const
{
	return mImg.size();
}

void ChainPlug::FxE::onPaint(paint_args& pa)
{
	pa.mPainter->drawPixmap(pa.mFrame.left(), pa.mFrame.top(), mImg);
}

void ChainPlug::FxE::mousePressEvent(const mouse_args&)
{
}

void ChainPlug::FxE::mouseMoveEvent(const mouse_args&)
{
}

void ChainPlug::FxE::mouseReleaseEvent(const mouse_args& down)
{
	if (down.mEntry != NULL) mNav.selectFx(mFxId);
}

void ChainPlug::FxE::cache(const int entryH, const Look& look)
{
	if (mLabel.size() < 1) return;
	int					w = entryH, h = entryH;
	if (w < 1 || h < 1) return;
	QImage*				img = new QImage(w, h, QImage::Format_RGB888);
	if (img == NULL) return;

	{
		QPainter		painter(img);
		// Clear the background
		painter.setPen(Qt::NoPen);
		painter.setBrush(look.mDashBgB); //.mAlphaB);
		painter.drawRect(0, 0, w, h);
		// Draw a param panel
		painter.setBrush(look.mParamBgB);
		painter.setRenderHint(QPainter::Antialiasing);
		painter.drawRoundedRect(0, 0, w, h, look.mRound, look.mRound);
		// Draw the label
		painter.setPen(look.mParamFgC);
		painter.drawText(4, 10, mLabel);
	}

	// Kinda weird -- where's the error check?
	mImg = QPixmap::fromImage(*img);
	delete img;
}
