#include "renderview.h"

#include <QtDebug>
#include <QtGui>

#include "io/svgreader.h"
#include "model/activeproject.h"

RenderView::RenderView(ActiveProject& prj, QWidget *parent) :
	inherited(parent) ,
	mPrj(prj) ,
	mEmpty(true)
{
	mDataSize = QSize(256, 256);

	connect(&mThread, SIGNAL(renderFinished()),
				 this, SLOT(onRenderFinished()));
}

QSize RenderView::minimumSizeHint() const
{
	return QSize(256, 256);
}

QSize RenderView::sizeHint() const
{
	return QSize(400, 256);
}

void RenderView::onProjectChanged(const int)
{
	// Replace the entire FX structure
	Project					prj;
	if (!mPrj.copyProject(prj)) return;
	if (mThread.renderProject(prj, mDataSize)) mEmpty = false;
}

void RenderView::onParamChanged(const Param& p)
{
	if (mEmpty) onProjectChanged(ActiveProject::STRUCTURE_CHANGED);
	else mThread.renderParam(p);
}

void RenderView::paintEvent(QPaintEvent*)
{
	if (!mPixmap.isNull()) {
		QPainter painter(this);
		painter.drawPixmap(0, 0, mPixmap);
	}
}

void RenderView::onRenderFinished()
{
	if (mThread.getOutput(mPixmap)) update();
}
