#ifndef SHAPEBROWSER_H
#define SHAPEBROWSER_H

#include <QWidget>
#include <QPixmap>

#include "app/environment.h"
#include "ui/look.h"
class Project;

/* Widget that displays and allows selections of shapes.  Shapes
 * start as SVG files, and get assigned to layers.
 */
class ShapeBrowser : public QWidget
{
    Q_OBJECT
public:
	ShapeBrowser(const Environment&, Project&, const Look&, QWidget *parent = 0);

	QSize minimumSizeHint() const;

signals:

public slots:

protected:
	void mouseReleaseEvent(QMouseEvent *);
	void paintEvent(QPaintEvent *);

private:
	class Entry;

	const Environment&	mEnv;
	Project&			mPrj;
	const Look&			mLook;
	Entry*				mHead;

	// METRICS
	int					mEntryH;
	int					mViewH;

	bool				mLayout;	// Set to true when I need a layout

	bool add(const QString& filename);
	void layoutEntries();

	class Entry {
	public:
		Entry*			mNext;
		QRect			mFrame;
		QString			mFilename;
		QPixmap			mImg;

		Entry();

		Entry*			tail();
		const Entry*	find(const int x, const int y) const;
		bool			setImg(const QString& filename, const int size, const Look&);
	};
};

#endif // SHAPEBROWSER_H
